# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2024 Valory AG
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This test module contains the tests for the `aea scaffold skill` sub-command."""

import json
import os
import re
import shutil
import subprocess  # nosec
import tempfile
import unittest.mock
from pathlib import Path
from typing import cast

import jsonschema
import yaml
from jsonschema import Draft4Validator, ValidationError

from aea import AEA_DIR
from aea.cli import cli
from aea.cli.packages import get_package_manager
from aea.configurations.base import DEFAULT_SKILL_CONFIG_FILE, DEFAULT_VERSION
from aea.configurations.constants import DEFAULT_AEA_CONFIG_FILE, PACKAGES, SKILL
from aea.configurations.data_types import PackageId
from aea.configurations.loader import make_jsonschema_base_uri
from aea.package_manager.v1 import PackageManagerV1

from tests.conftest import (
    AUTHOR,
    CLI_LOG_OPTION,
    CONFIGURATION_SCHEMA_DIR,
    CliRunner,
    ROOT_DIR,
    SKILL_CONFIGURATION_SCHEMA,
)
from tests.test_cli.test_scaffold import files_outside_copyright_are_identical


class TestScaffoldSkill:
    """Test that the command 'aea scaffold skill' works correctly in correct preconditions."""

    @classmethod
    def setup_class(cls):
        """Set the test up."""
        cls.runner = CliRunner()
        cls.agent_name = "myagent"
        cls.resource_name = "myresource"
        cls.cwd = os.getcwd()
        cls.t = tempfile.mkdtemp()
        dir_path = Path("packages")
        tmp_dir = cls.t / dir_path
        src_dir = cls.cwd / Path(ROOT_DIR, dir_path)
        shutil.copytree(str(src_dir), str(tmp_dir))

        cls.schema = json.load(open(SKILL_CONFIGURATION_SCHEMA))
        cls.resolver = jsonschema.RefResolver(
            make_jsonschema_base_uri(Path(CONFIGURATION_SCHEMA_DIR).absolute()),
            cls.schema,
        )
        cls.validator = Draft4Validator(cls.schema, resolver=cls.resolver)

        os.chdir(cls.t)
        result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "init", "--local", "--author", AUTHOR],
        )
        assert result.exit_code == 0
        result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "create", "--local", cls.agent_name],
            standalone_mode=False,
        )
        assert result.exit_code == 0
        os.chdir(cls.agent_name)
        # scaffold skill
        cls.result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "scaffold", "skill", cls.resource_name],
            standalone_mode=False,
        )

    def test_exit_code_equal_to_0(self):
        """Test that the exit code is equal to 0."""
        assert self.result.exit_code == 0

    def test_resource_folder_contains_module_handlers(self):
        """Test that the resource folder contains scaffold handlers.py module."""
        p = Path(self.t, self.agent_name, "skills", self.resource_name, "handlers.py")
        original = Path(AEA_DIR, "skills", "scaffold", "handlers.py")
        assert files_outside_copyright_are_identical(p, original)

    def test_resource_folder_contains_module_behaviours(self):
        """Test that the resource folder contains scaffold behaviours.py module."""
        p = Path(self.t, self.agent_name, "skills", self.resource_name, "behaviours.py")
        original = Path(AEA_DIR, "skills", "scaffold", "behaviours.py")
        assert files_outside_copyright_are_identical(p, original)

    def test_resource_folder_contains_module_model(self):
        """Test that the resource folder contains scaffold my_model.py module."""
        p = Path(self.t, self.agent_name, "skills", self.resource_name, "my_model.py")
        original = Path(AEA_DIR, "skills", "scaffold", "my_model.py")
        assert files_outside_copyright_are_identical(p, original)

    def test_resource_folder_contains_configuration_file(self):
        """Test that the resource folder contains a good configuration file."""
        p = Path(
            self.t,
            self.agent_name,
            "skills",
            self.resource_name,
            DEFAULT_SKILL_CONFIG_FILE,
        )
        config_file = yaml.safe_load(open(p))
        self.validator.validate(instance=config_file)

    def test_init_module_contains_new_public_id(self):
        """Test that the PUBLIC ID variable in the init module is replaced correctly."""
        p = Path(self.t, self.agent_name, "skills", self.resource_name, "__init__.py")
        init_module_content = p.read_text()
        expected_public_id = f"{AUTHOR}/{self.resource_name}:{DEFAULT_VERSION}"
        matches = re.findall(
            rf'^PUBLIC_ID = PublicId\.from_str\("{expected_public_id}"\)$',
            init_module_content,
            re.MULTILINE,
        )
        assert len(matches) == 1

    @classmethod
    def teardown_class(cls):
        """Tear the test down."""
        os.chdir(cls.cwd)
        try:
            shutil.rmtree(cls.t)
        except (OSError, IOError):
            pass


class TestScaffoldSkillToRegistry:
    """Test that the command 'aea scaffold skill' works correctly in correct preconditions."""

    @classmethod
    def setup_class(cls):
        """Set the test up."""
        cls.runner = CliRunner()
        cls.agent_name = "myagent"
        cls.resource_name = "myresource"
        cls.cwd = os.getcwd()
        cls.t = tempfile.mkdtemp()
        dir_path = Path("packages")
        tmp_dir = cls.t / dir_path
        src_dir = cls.cwd / Path(ROOT_DIR, dir_path)
        shutil.copytree(str(src_dir), str(tmp_dir))

        cls.schema = json.load(open(SKILL_CONFIGURATION_SCHEMA))
        cls.resolver = jsonschema.RefResolver(
            make_jsonschema_base_uri(Path(CONFIGURATION_SCHEMA_DIR).absolute()),
            cls.schema,
        )
        cls.validator = Draft4Validator(cls.schema, resolver=cls.resolver)

        os.chdir(cls.t)
        result = cls.runner.invoke(
            cli, [*CLI_LOG_OPTION, "init", "--local", "--author", AUTHOR]
        )
        assert result.exit_code == 0

        dir_path.mkdir(exist_ok=True)
        # scaffold skill
        cls.result = cls.runner.invoke(
            cli,
            [
                *CLI_LOG_OPTION,
                f"--registry-path={str(dir_path)}",
                "scaffold",
                "--to-local-registry",
                "skill",
                cls.resource_name,
            ],
            standalone_mode=False,
        )

    def test_exit_code_equal_to_0(self):
        """Test that the exit code is equal to 0."""
        assert self.result.exit_code == 0

    def test_resource_folder_contains_module_handlers(self):
        """Test that the resource folder contains scaffold handlers.py module."""
        p = Path(
            self.t, "packages", AUTHOR, "skills", self.resource_name, "handlers.py"
        )
        original = Path(AEA_DIR, "skills", "scaffold", "handlers.py")
        assert files_outside_copyright_are_identical(p, original)

    def test_resource_folder_contains_module_behaviours(self):
        """Test that the resource folder contains scaffold behaviours.py module."""
        p = Path(
            self.t, "packages", AUTHOR, "skills", self.resource_name, "behaviours.py"
        )
        original = Path(AEA_DIR, "skills", "scaffold", "behaviours.py")
        assert files_outside_copyright_are_identical(p, original)

    def test_resource_folder_contains_module_model(self):
        """Test that the resource folder contains scaffold my_model.py module."""
        p = Path(
            self.t, "packages", AUTHOR, "skills", self.resource_name, "my_model.py"
        )
        original = Path(AEA_DIR, "skills", "scaffold", "my_model.py")
        assert files_outside_copyright_are_identical(p, original)

    def test_resource_folder_contains_configuration_file(self):
        """Test that the resource folder contains a good configuration file."""
        p = Path(
            self.t,
            "packages",
            AUTHOR,
            "skills",
            self.resource_name,
            DEFAULT_SKILL_CONFIG_FILE,
        )
        config_file = yaml.safe_load(open(p))
        self.validator.validate(instance=config_file)

    def test_init_module_contains_new_public_id(self):
        """Test that the PUBLIC ID variable in the init module is replaced correctly."""
        p = Path(
            self.t, "packages", AUTHOR, "skills", self.resource_name, "__init__.py"
        )
        init_module_content = p.read_text()
        expected_public_id = f"{AUTHOR}/{self.resource_name}:{DEFAULT_VERSION}"
        matches = re.findall(
            rf'^PUBLIC_ID = PublicId\.from_str\("{expected_public_id}"\)$',
            init_module_content,
            re.MULTILINE,
        )
        assert len(matches) == 1

    def test_the_package_is_registered(self):
        """Test that the package is already registered."""
        pm = cast(
            PackageManagerV1,
            get_package_manager(
                package_dir=Path(
                    self.t,
                    "packages",
                )
            ),
        )
        assert pm.is_dev_package(
            package_id=PackageId.from_uri_path(
                f"skill/{AUTHOR}/{self.resource_name}/0.1.0"
            )
        )

    @classmethod
    def teardown_class(cls):
        """Tear the test down."""
        os.chdir(cls.cwd)
        try:
            shutil.rmtree(cls.t)
        except (OSError, IOError):
            pass


class TestScaffoldSkillFailsWhenDirectoryAlreadyExists:
    """Test that the command 'aea scaffold skill' fails when a folder with 'scaffold' name already."""

    @classmethod
    def setup_class(cls):
        """Set the test up."""
        cls.runner = CliRunner()
        cls.agent_name = "myagent"
        cls.resource_name = "myresource"
        cls.cwd = os.getcwd()
        cls.t = tempfile.mkdtemp()
        dir_path = Path("packages")
        tmp_dir = cls.t / dir_path
        src_dir = cls.cwd / Path(ROOT_DIR, dir_path)
        shutil.copytree(str(src_dir), str(tmp_dir))

        os.chdir(cls.t)
        result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "init", "--local", "--author", AUTHOR],
        )
        assert result.exit_code == 0
        result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "create", "--local", cls.agent_name],
            standalone_mode=False,
        )
        assert result.exit_code == 0
        os.chdir(cls.agent_name)
        # create a dummy 'myresource' folder
        Path(cls.t, cls.agent_name, "skills", cls.resource_name).mkdir(
            exist_ok=False, parents=True
        )
        cls.result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "scaffold", "skill", cls.resource_name],
            standalone_mode=False,
        )

    def test_exit_code_equal_to_1(self):
        """Test that the exit code is equal to 1 (i.e. catchall for general errors)."""
        assert self.result.exit_code == 1

    def test_error_message_skill_already_existing(self):
        """Test that the log error message is fixed.

        The expected message is: 'A skill with name '{skill_name}' already exists. Aborting...'
        """
        s = "A skill with this name already exists. Please choose a different name and try again."
        assert self.result.exception.message == s

    def test_resource_directory_exists(self):
        """Test that the resource directory still exists.

        This means that after every failure, we make sure we restore the previous state.
        """
        assert Path(self.t, self.agent_name, "skills", self.resource_name).exists()

    @classmethod
    def teardown_class(cls):
        """Tear the test down."""
        os.chdir(cls.cwd)
        try:
            shutil.rmtree(cls.t)
        except (OSError, IOError):
            pass


class TestScaffoldSkillFailsWhenSkillAlreadyExists:
    """Test that the command 'aea add skill' fails when the skill already exists."""

    @classmethod
    def setup_class(cls):
        """Set the test up."""
        cls.runner = CliRunner()
        cls.agent_name = "myagent"
        cls.resource_name = "myresource"
        cls.cwd = os.getcwd()
        cls.t = tempfile.mkdtemp()
        dir_path = Path("packages")
        tmp_dir = cls.t / dir_path
        src_dir = cls.cwd / Path(ROOT_DIR, dir_path)
        shutil.copytree(str(src_dir), str(tmp_dir))

        os.chdir(cls.t)
        result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "init", "--local", "--author", AUTHOR],
        )
        assert result.exit_code == 0
        result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "create", "--local", cls.agent_name],
            standalone_mode=False,
        )
        assert result.exit_code == 0
        os.chdir(cls.agent_name)
        # add skill first time
        result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "scaffold", "skill", cls.resource_name],
            standalone_mode=False,
        )
        assert result.exit_code == 0
        # scaffold skill with the same skill name
        cls.result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "scaffold", "skill", cls.resource_name],
            standalone_mode=False,
        )

    def test_exit_code_equal_to_1(self):
        """Test that the exit code is equal to 1 (i.e. catchall for general errors)."""
        assert self.result.exit_code == 1

    def test_error_message_skill_already_existing(self):
        """Test that the log error message is fixed.

        The expected message is: 'A skill with name '{skill_name}' already exists. Aborting...'
        """
        s = "A skill with name '{}' already exists. Aborting...".format(
            self.resource_name
        )
        assert self.result.exception.message == s

    def test_resource_directory_exists(self):
        """Test that the resource directory still exists.

        This means that after every failure, we make sure we restore the previous state.
        """
        assert Path(self.t, self.agent_name, "skills", self.resource_name).exists()

    @classmethod
    def teardown_class(cls):
        """Tear the test down."""
        os.chdir(cls.cwd)
        try:
            shutil.rmtree(cls.t)
        except (OSError, IOError):
            pass


class TestScaffoldSkillFailsWhenConfigFileIsNotCompliant:
    """Test that the command 'aea scaffold skill' fails when the configuration file is not compliant with the schema."""

    @classmethod
    def setup_class(cls):
        """Set the test up."""
        cls.runner = CliRunner()
        cls.agent_name = "myagent"
        cls.resource_name = "myresource"
        cls.cwd = os.getcwd()
        cls.t = tempfile.mkdtemp()
        dir_path = Path("packages")
        tmp_dir = cls.t / dir_path
        src_dir = cls.cwd / Path(ROOT_DIR, dir_path)
        shutil.copytree(str(src_dir), str(tmp_dir))

        os.chdir(cls.t)
        result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "init", "--local", "--author", AUTHOR],
        )
        assert result.exit_code == 0
        result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "create", "--local", cls.agent_name],
            standalone_mode=False,
        )
        assert result.exit_code == 0

        # change the dumping of yaml module to raise an exception.
        cls.patch = unittest.mock.patch(
            "yaml.dump",
            side_effect=ValidationError("test error message"),
        )
        cls.patch.start()

        os.chdir(cls.agent_name)
        cls.result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "scaffold", "skill", cls.resource_name],
            standalone_mode=False,
        )

    def test_exit_code_equal_to_1(self):
        """Test that the exit code is equal to 1 (i.e. catchall for general errors)."""
        assert self.result.exit_code == 1

    def test_configuration_file_not_valid(self):
        """Test that the log error message is fixed.

        The expected message is: 'Cannot find skill: '{skill_name}'
        """
        s = "Error when validating the skill configuration file."
        assert self.result.exception.message == s

    def test_resource_directory_does_not_exists(self):
        """Test that the resource directory does not exist.

        This means that after every failure, we make sure we restore the previous state.
        """
        assert not Path(self.t, self.agent_name, "skills", self.resource_name).exists()

    @classmethod
    def teardown_class(cls):
        """Tear the test down."""
        cls.patch.stop()
        os.chdir(cls.cwd)
        try:
            shutil.rmtree(cls.t)
        except (OSError, IOError):
            pass


class TestScaffoldSkillFailsWhenExceptionOccurs:
    """Test that the command 'aea scaffold skill' fails when the configuration file is not compliant with the schema."""

    @classmethod
    def setup_class(cls):
        """Set the test up."""
        cls.runner = CliRunner()
        cls.agent_name = "myagent"
        cls.resource_name = "myresource"
        cls.cwd = os.getcwd()
        cls.t = tempfile.mkdtemp()
        dir_path = Path("packages")
        tmp_dir = cls.t / dir_path
        src_dir = cls.cwd / Path(ROOT_DIR, dir_path)
        shutil.copytree(str(src_dir), str(tmp_dir))

        os.chdir(cls.t)
        result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "init", "--local", "--author", AUTHOR],
        )
        assert result.exit_code == 0
        result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "create", "--local", cls.agent_name],
            standalone_mode=False,
        )
        assert result.exit_code == 0

        cls.patch = unittest.mock.patch(
            "shutil.copytree", side_effect=Exception("unknwon exception")
        )
        cls.patch.start()

        os.chdir(cls.agent_name)
        cls.result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "scaffold", "skill", cls.resource_name],
            standalone_mode=False,
        )

    def test_exit_code_equal_to_1(self):
        """Test that the exit code is equal to 1 (i.e. catchall for general errors)."""
        assert self.result.exit_code == 1

    def test_resource_directory_does_not_exists(self):
        """Test that the resource directory does not exist.

        This means that after every failure, we make sure we restore the previous state.
        """
        assert not Path(self.t, self.agent_name, "skills", self.resource_name).exists()

    @classmethod
    def teardown_class(cls):
        """Tear the test down."""
        cls.patch.stop()
        os.chdir(cls.cwd)
        try:
            shutil.rmtree(cls.t)
        except (OSError, IOError):
            pass


class TestScaffoldSkillHelpWithInvalidDirectory:
    """Test --help works when validation fails."""

    @classmethod
    def setup_class(cls):
        """Set the test up."""
        cls.runner = CliRunner()
        cls.agent_name = "myagent"
        cls.resource_name = "myresource"
        cls.cwd = os.getcwd()
        cls.t = tempfile.mkdtemp()
        dir_path = Path(PACKAGES)
        tmp_dir = cls.t / dir_path
        src_dir = cls.cwd / Path(ROOT_DIR, dir_path)
        shutil.copytree(str(src_dir), str(tmp_dir))

        os.chdir(cls.t)
        result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "init", "--local", "--author", AUTHOR],
        )
        assert result.exit_code == 0
        result = cls.runner.invoke(
            cli,
            [*CLI_LOG_OPTION, "create", "--local", cls.agent_name],
            standalone_mode=False,
        )
        assert result.exit_code == 0

        os.remove(Path(cls.t) / cls.agent_name / DEFAULT_AEA_CONFIG_FILE)

    def test_exit_code_equal_to_0(self):
        """Test that --help works."""
        result = subprocess.run(  # nosec
            [
                "aea",
                "scaffold",
                SKILL,
                "--help",
            ],
            cwd=self.t,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        assert result.returncode == 0, result.stderr

    @classmethod
    def teardown_class(cls):
        """Tear the test down."""
        os.chdir(cls.cwd)
        try:
            shutil.rmtree(cls.t)
        except (OSError, IOError):
            pass



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2025 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test errors."""

# pylint: skip-file

import asyncio
import os
import platform
import subprocess  # nosec
import sys
import tempfile
from asyncio.futures import Future
from unittest.mock import Mock, patch

import pytest

from aea.configurations.base import ConnectionConfig
from aea.multiplexer import Multiplexer

from packages.valory.connections.p2p_libp2p.connection import (
    LIBP2P_NODE_MODULE_NAME,
    Libp2pNode,
    P2PLibp2pConnection,
    _golang_module_run,
    _ip_all_private_or_all_public,
)
from packages.valory.connections.p2p_libp2p.tests.base import ports
from packages.valory.connections.test_libp2p.tests.base import (
    BaseP2PLibp2pTest,
    SKIP_WINDOWS,
    _make_libp2p_connection,
    create_identity,
)
from packages.valory.protocols.acn.message import AcnMessage


class TestP2PLibp2pConnectionFailureGolangRun(BaseP2PLibp2pTest):
    """Test that golang run fails if wrong path or timeout"""

    def test_wrong_path(self) -> None:
        """Test the wrong path."""
        log_file_desc = open("log", "a", 1)
        wrong_path = tempfile.mkdtemp()
        with pytest.raises(FileNotFoundError):  # match differs based on OS
            _golang_module_run(wrong_path, LIBP2P_NODE_MODULE_NAME, [], log_file_desc)

    def test_timeout(self) -> None:
        """Test the timeout."""

        connection = _make_libp2p_connection()
        connection.node._connection_timeout = 0
        muxer = Multiplexer([connection])
        with pytest.raises(Exception, match="Couldn't connect to libp2p process"):
            muxer.connect()
        muxer.disconnect()


class TestP2PLibp2pConnectionFailureSetupNewConnection(BaseP2PLibp2pTest):
    """Test that connection constructor ensures proper configuration"""

    key_file: str

    @classmethod
    def setup_class(cls) -> None:
        """Set the test up"""

        super().setup_class()
        crypto = cls.default_crypto
        cls.identity = create_identity(crypto)
        cls.host = "localhost"
        cls.port = next(ports)
        cls.key_file = os.path.join(cls.tmp, "keyfile")
        crypto.dump(cls.key_file)

    def test_entry_peers_when_no_public_uri_provided(self) -> None:
        """Test entry peers when no public uri provided."""

        configuration = ConnectionConfig(
            libp2p_key_file=None,
            local_uri=f"{self.host}:{self.port}",
            delegate_uri=f"{self.host}:{self.port}",
            entry_peers=None,
            log_file=None,
            connection_id=P2PLibp2pConnection.connection_id,
            build_directory=self.tmp,
        )
        with pytest.raises(ValueError, match="Couldn't find connection key"):
            P2PLibp2pConnection(
                configuration=configuration, data_dir=self.tmp, identity=self.identity
            )

    def test_local_uri_provided_when_public_uri_provided(self) -> None:
        """Test local uri provided when public uri provided."""

        configuration = ConnectionConfig(
            node_key_file=self.key_file,
            public_uri=f"{self.host}:{self.port}",
            entry_peers=None,
            log_file=None,
            connection_id=P2PLibp2pConnection.connection_id,
            build_directory=self.tmp,
        )
        with pytest.raises(ValueError, match="Couldn't find connection key"):
            P2PLibp2pConnection(
                configuration=configuration, data_dir=self.tmp, identity=self.identity
            )


@SKIP_WINDOWS
def test_libp2p_connection_mixed_ip_address() -> None:
    """Test correct public uri ip and entry peers ips configuration."""
    assert _ip_all_private_or_all_public([]) is True
    assert _ip_all_private_or_all_public(["127.0.0.1", "127.0.0.1"]) is True
    assert _ip_all_private_or_all_public(["localhost", "127.0.0.1"]) is True
    assert _ip_all_private_or_all_public(["10.0.0.1", "127.0.0.1"]) is False
    assert _ip_all_private_or_all_public(["fetch.ai", "127.0.0.1"]) is False
    assert _ip_all_private_or_all_public(["104.26.2.97", "127.0.0.1"]) is False
    assert _ip_all_private_or_all_public(["fetch.ai", "acn.fetch.ai"]) is True


@pytest.mark.skipif(
    condition=(platform.system() == "Windows"),
    reason="https://github.com/golang/go/issues/51007",
)
@SKIP_WINDOWS
def test_libp2p_connection_node_config_registration_delay() -> None:
    """Test node registration delay configuration"""

    with pytest.raises(ValueError, match="must be a float number in seconds"):
        _make_libp2p_connection(peer_registration_delay="must_be_float")


@SKIP_WINDOWS
def test_build_dir_not_set() -> None:
    """Test build dir not set."""

    with tempfile.TemporaryDirectory() as data_dir:
        con = _make_libp2p_connection(data_dir=data_dir)
        con.configuration.build_directory = None
        with pytest.raises(
            ValueError, match="Connection Configuration build directory is not set!"
        ):
            P2PLibp2pConnection(
                configuration=con.configuration,
                data_dir=data_dir,
                identity=con._identity,
                crypto_store=con.crypto_store,
            )


@pytest.mark.asyncio
@SKIP_WINDOWS
async def test_reconnect_on_write_failed() -> None:
    """Test node restart on write fail."""

    con = _make_libp2p_connection()
    node = Libp2pNode(Mock(), Mock(), "tmp", "tmp")
    con.node = node
    node.pipe = Mock()
    node.pipe.write = Mock(side_effect=Exception("expected"))
    con._node_client = node.get_client()
    f = Future()
    f.set_result(None)
    with patch.object(
        con, "_restart_node", return_value=f
    ) as restart_mock, patch.object(
        con, "_ensure_valid_envelope_for_external_comms"
    ), patch.object(
        con._node_client, "make_acn_envelope_message", return_value=b"some_data"
    ), pytest.raises(
        Exception, match="expected"
    ):
        await con._send_envelope_with_node_client(Mock())

    assert node.pipe.write.call_count == 2
    restart_mock.assert_called_once()


@pytest.mark.asyncio
@SKIP_WINDOWS
async def test_reconnect_on_write_failed_reconnect_pipe() -> None:
    """Test node restart on write fail."""

    con = _make_libp2p_connection()

    node = Libp2pNode(Mock(), Mock(), "tmp", "tmp")
    f = Future()
    f.set_result(None)
    con.node = node
    node.pipe = Mock()
    node.pipe.connect = Mock(return_value=f)
    node.pipe.write = Mock(side_effect=[Exception("expected"), f])
    node.pipe.close = Mock(return_value=f)

    con._node_client = node.get_client()
    status_ok = Mock()
    status_ok.code = int(AcnMessage.StatusBody.StatusCode.SUCCESS)
    status_ok_future = Future()
    status_ok_future.set_result(status_ok)
    with patch.object(con, "_ensure_valid_envelope_for_external_comms"), patch.object(
        node, "is_proccess_running", return_value=True
    ), patch.object(
        con._node_client, "make_acn_envelope_message", return_value=b"some_data"
    ), patch.object(
        con._node_client, "wait_for_status", lambda: status_ok_future
    ):
        await con._send_envelope_with_node_client(Mock())

    assert node.pipe.write.call_count == 2
    assert node.pipe.connect.call_count == 1


@pytest.mark.asyncio
@SKIP_WINDOWS
async def test_reconnect_on_read_failed() -> None:
    """Test node restart on read fail."""

    con = _make_libp2p_connection()
    node = Libp2pNode(Mock(), Mock(), "tmp", "tmp")
    con.node = node
    node.pipe = Mock()
    node.pipe.read = Mock(side_effect=Exception("expected"))
    con._node_client = node.get_client()
    f = Future()
    f.set_result(None)
    with patch.object(
        con, "_restart_node", return_value=f
    ) as restart_mock, pytest.raises(Exception, match="expected"):
        await con._read_envelope_from_node()

    assert node.pipe.read.call_count == 2
    restart_mock.assert_called_once()

    assert node.pipe.read.call_count == 2
    restart_mock.assert_called_once()


@pytest.mark.asyncio
@SKIP_WINDOWS
async def test_node_stopped_callback() -> None:
    """Test node stopped callback called."""

    if not (
        platform.system() != "Windows"
        and sys.version_info.major == 3
        and sys.version_info.minor >= 10
    ):
        pytest.skip(
            "Not supported on this platform. Unix and python >= 3.10 supported only"
        )

    con = _make_libp2p_connection()
    con.node.logger.error = Mock()
    await con.node.start()
    subprocess.Popen.terminate(con.node.proc)
    await asyncio.sleep(2)
    con.node.logger.error.assert_called_once()
    await con.node.stop()

    con = _make_libp2p_connection()
    con.node.logger.error = Mock()
    await con.node.start()
    await con.node.stop()
    await asyncio.sleep(2)
    con.node.logger.error.assert_not_called()



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2026 Valory AG
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains the tests of the handler classes of the erc1155_deploy skill."""

# pylint: skip-file

import logging
from typing import cast
from unittest.mock import patch

from aea.crypto.ledger_apis import LedgerApis
from aea.helpers.transaction.base import State
from aea.protocols.dialogue.base import Dialogues
from aea.test_tools.test_skill import COUNTERPARTY_AGENT_ADDRESS

from packages.fetchai.protocols.default.message import DefaultMessage
from packages.fetchai.protocols.fipa.message import FipaMessage
from packages.fetchai.protocols.oef_search.message import OefSearchMessage
from packages.fetchai.skills.erc1155_deploy.dialogues import (
    ContractApiDialogue,
    FipaDialogue,
    LedgerApiDialogue,
    OefSearchDialogue,
    SigningDialogue,
)
from packages.fetchai.skills.erc1155_deploy.handlers import LEDGER_API_ADDRESS
from packages.fetchai.skills.erc1155_deploy.tests.intermediate_class import (
    ERC1155DeployTestCase,
)
from packages.open_aea.protocols.signing.message import SigningMessage
from packages.valory.protocols.contract_api.message import ContractApiMessage
from packages.valory.protocols.ledger_api.message import LedgerApiMessage


class TestFipaHandler(ERC1155DeployTestCase):
    """Test fipa handler of erc1155_deploy."""

    def test_setup(self):
        """Test the setup method of the fipa handler."""
        assert self.fipa_handler.setup() is None
        self.assert_quantity_in_outbox(0)

    def test_handle_unidentified_dialogue(self):
        """Test the _handle_unidentified_dialogue method of the fipa handler."""
        # setup
        incorrect_dialogue_reference = ("", "")
        incoming_message = cast(
            FipaMessage,
            self.build_incoming_message(
                message_type=FipaMessage,
                dialogue_reference=incorrect_dialogue_reference,
                performative=FipaMessage.Performative.ACCEPT,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.fipa_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"unidentified dialogue for message={incoming_message}.",
        )

        self.assert_quantity_in_outbox(1)
        has_attributes, error_str = self.message_has_attributes(
            actual_message=self.get_message_from_outbox(),
            message_type=DefaultMessage,
            performative=DefaultMessage.Performative.ERROR,
            to=incoming_message.sender,
            sender=self.skill.skill_context.agent_address,
            error_code=DefaultMessage.ErrorCode.INVALID_DIALOGUE,
            error_msg="Invalid dialogue.",
            error_data={"fipa_message": incoming_message.encode()},
        )
        assert has_attributes, error_str

    def test_handle_cfp_i(self):
        """Test the _handle_cfp method of the fipa handler where is_tokens_minted is True."""
        # setup
        self.strategy._is_tokens_minted = True
        incoming_message = cast(
            FipaMessage,
            self.build_incoming_message(
                message_type=FipaMessage,
                performative=FipaMessage.Performative.CFP,
                dialogue_reference=Dialogues.new_self_initiated_dialogue_reference(),
                query=self.mocked_query,
            ),
        )

        # operation
        with patch.object(
            self.strategy, "get_proposal", return_value=self.mocked_proposal
        ) as mock_prop:
            with patch.object(self.logger, "log") as mock_logger:
                self.fipa_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received CFP from sender={COUNTERPARTY_AGENT_ADDRESS[-5:]}",
        )

        mock_prop.assert_called_once()

        self.assert_quantity_in_outbox(1)
        message = self.get_message_from_outbox()
        has_attributes, error_str = self.message_has_attributes(
            actual_message=message,
            message_type=FipaMessage,
            performative=FipaMessage.Performative.PROPOSE,
            to=incoming_message.sender,
            sender=self.skill.skill_context.agent_address,
            proposal=self.mocked_proposal,
        )
        assert has_attributes, error_str

        mock_logger.assert_any_call(
            logging.INFO,
            f"sending PROPOSE to agent={COUNTERPARTY_AGENT_ADDRESS[-5:]}: proposal={self.mocked_proposal.values}",
        )

    def test_handle_cfp_ii(self):
        """Test the _handle_cfp method of the fipa handler where is_tokens_minted is False."""
        # setup
        self.strategy._is_tokens_minted = False
        incoming_message = cast(
            FipaMessage,
            self.build_incoming_message(
                message_type=FipaMessage,
                performative=FipaMessage.Performative.CFP,
                dialogue_reference=Dialogues.new_self_initiated_dialogue_reference(),
                query=self.mocked_query,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.fipa_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received CFP from sender={COUNTERPARTY_AGENT_ADDRESS[-5:]}",
        )

        mock_logger.assert_any_call(
            logging.INFO,
            "Contract items not minted yet. Try again later.",
        )

        self.assert_quantity_in_outbox(0)

    def test_handle_accept_w_inform_i(self):
        """Test the _handle_accept_w_inform method of the fipa handler where tx_signature is NOT None."""
        # setup
        tx_signature = "some_tx_signature"
        self.strategy.contract_address = self.contract_address
        fipa_dialogue = cast(
            FipaDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.fipa_dialogues,
                messages=self.list_of_fipa_messages[:2],
            ),
        )
        fipa_dialogue.proposal = self.mocked_proposal
        incoming_message = cast(
            FipaMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=fipa_dialogue,
                performative=FipaMessage.Performative.ACCEPT_W_INFORM,
                info={"tx_signature": tx_signature},
            ),
        )

        # operation
        with patch.object(
            self.strategy, "get_single_swap_terms", return_value=self.mocked_terms
        ) as mock_swap:
            with patch.object(self.logger, "log") as mock_logger:
                self.fipa_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received ACCEPT_W_INFORM from sender={COUNTERPARTY_AGENT_ADDRESS[-5:]}: tx_signature={tx_signature}",
        )

        self.assert_quantity_in_outbox(1)
        message = self.get_message_from_outbox()
        has_attributes, error_str = self.message_has_attributes(
            actual_message=message,
            message_type=ContractApiMessage,
            performative=ContractApiMessage.Performative.GET_RAW_TRANSACTION,
            to=LEDGER_API_ADDRESS,
            sender=str(self.skill.skill_context.skill_id),
            ledger_id=self.strategy.ledger_id,
            contract_id=self.strategy.contract_id,
            contract_address=self.strategy.contract_address,
            callable="get_atomic_swap_single_transaction",
            kwargs=ContractApiMessage.Kwargs(
                {
                    "from_address": self.skill.skill_context.agent_address,
                    "to_address": incoming_message.sender,
                    "token_id": int(fipa_dialogue.proposal.values["token_id"]),
                    "from_supply": int(fipa_dialogue.proposal.values["from_supply"]),
                    "to_supply": int(fipa_dialogue.proposal.values["to_supply"]),
                    "value": int(fipa_dialogue.proposal.values["value"]),
                    "trade_nonce": int(fipa_dialogue.proposal.values["trade_nonce"]),
                    "signature": tx_signature,
                }
            ),
        )
        assert has_attributes, error_str

        mock_swap.assert_called_once()
        contract_api_dialogue = cast(
            ContractApiDialogue, self.contract_api_dialogues.get_dialogue(message)
        )
        assert contract_api_dialogue.terms == self.mocked_terms

        mock_logger.assert_any_call(
            logging.INFO,
            "requesting single atomic swap transaction...",
        )

    def test_handle_accept_w_inform_ii(self):
        """Test the _handle_accept_w_inform method of the fipa handler where tx_signature is NOT None."""
        # setup
        tx_signature = "some_tx_signature"
        fipa_dialogue = cast(
            FipaDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.fipa_dialogues,
                messages=self.list_of_fipa_messages[:2],
            ),
        )
        fipa_dialogue.proposal = self.mocked_proposal
        incoming_message = cast(
            FipaMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=fipa_dialogue,
                performative=FipaMessage.Performative.ACCEPT_W_INFORM,
                info={"something": tx_signature},
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.fipa_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received ACCEPT_W_INFORM from sender={COUNTERPARTY_AGENT_ADDRESS[-5:]} with no signature.",
        )

    def test_handle_invalid(self):
        """Test the _handle_invalid method of the fipa handler."""
        # setup
        fipa_dialogue = cast(
            FipaDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.fipa_dialogues,
                messages=self.list_of_fipa_messages[:2],
            ),
        )
        incoming_message = cast(
            FipaMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=fipa_dialogue,
                performative=FipaMessage.Performative.ACCEPT,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.fipa_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.WARNING,
            f"cannot handle fipa message of performative={incoming_message.performative} in dialogue={fipa_dialogue}.",
        )

    def test_teardown(self):
        """Test the teardown method of the fipa handler."""
        assert self.fipa_handler.teardown() is None
        self.assert_quantity_in_outbox(0)


class TestLedgerApiHandler(ERC1155DeployTestCase):
    """Test ledger_api handler of erc1155_deploy."""

    is_agent_to_agent_messages = False

    def test_setup(self):
        """Test the setup method of the ledger_api handler."""
        assert self.ledger_api_handler.setup() is None
        self.assert_quantity_in_outbox(0)

    def test_handle_unidentified_dialogue(self):
        """Test the _handle_unidentified_dialogue method of the ledger_api handler."""
        # setup
        incorrect_dialogue_reference = ("", "")
        incoming_message = cast(
            LedgerApiMessage,
            self.build_incoming_message(
                message_type=LedgerApiMessage,
                dialogue_reference=incorrect_dialogue_reference,
                performative=LedgerApiMessage.Performative.BALANCE,
                ledger_id=self.ledger_id,
                balance=10,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.ledger_api_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received invalid ledger_api message={incoming_message}, unidentified dialogue.",
        )

    def test_handle_balance(self):
        """Test the _handle_balance method of the ledger_api handler."""
        # setup
        balance = 10
        ledger_api_dialogue = cast(
            LedgerApiDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.ledger_api_dialogues,
                messages=self.list_of_ledger_api_balance_messages[:1],
                counterparty=LEDGER_API_ADDRESS,
            ),
        )
        incoming_message = cast(
            LedgerApiMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=ledger_api_dialogue,
                performative=LedgerApiMessage.Performative.BALANCE,
                ledger_id=self.ledger_id,
                balance=balance,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.ledger_api_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"starting balance on {self.ledger_id} ledger={incoming_message.balance}.",
        )

    def test_handle_transaction_digest(self):
        """Test the _handle_transaction_digest method of the ledger_api handler."""
        # setup
        ledger_api_dialogue = cast(
            LedgerApiDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.ledger_api_dialogues,
                messages=self.list_of_ledger_api_messages[2:3],
                counterparty=LEDGER_API_ADDRESS,
            ),
        )
        incoming_message = cast(
            LedgerApiMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=ledger_api_dialogue,
                performative=LedgerApiMessage.Performative.TRANSACTION_DIGEST,
                transaction_digest=self.mocked_tx_digest,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.ledger_api_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"transaction was successfully submitted. Transaction digest={incoming_message.transaction_digest}",
        )

        self.assert_quantity_in_outbox(1)
        has_attributes, error_str = self.message_has_attributes(
            actual_message=self.get_message_from_outbox(),
            message_type=LedgerApiMessage,
            performative=LedgerApiMessage.Performative.GET_TRANSACTION_RECEIPT,
            to=incoming_message.sender,
            sender=str(self.skill.skill_context.skill_id),
            transaction_digest=self.mocked_tx_digest,
        )
        assert has_attributes, error_str

        mock_logger.assert_any_call(
            logging.INFO,
            "requesting transaction receipt.",
        )

    def test_handle_transaction_receipt_i(self):
        """Test the _handle_transaction_receipt method of the ledger_api handler where the transaction is NOT settled."""
        # setup
        ledger_api_dialogue = cast(
            LedgerApiDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.ledger_api_dialogues,
                messages=self.list_of_ledger_api_messages[4:5],
                counterparty=LEDGER_API_ADDRESS,
            ),
        )
        incoming_message = cast(
            LedgerApiMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=ledger_api_dialogue,
                performative=LedgerApiMessage.Performative.TRANSACTION_RECEIPT,
                transaction_receipt=self.mocked_tx_receipt,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            with patch.object(LedgerApis, "is_transaction_settled", return_value=False):
                self.ledger_api_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.ERROR,
            f"transaction failed. Transaction receipt={incoming_message.transaction_receipt}",
        )

    def test_handle_transaction_receipt_ii(self):
        """Test the _handle_transaction_receipt method of the ledger_api handler where is_contract_deployed is False."""
        # setup
        self.strategy._is_contract_deployed = False

        ledger_api_dialogue = cast(
            LedgerApiDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.ledger_api_dialogues,
                messages=self.list_of_ledger_api_messages[4:5],
                counterparty=LEDGER_API_ADDRESS,
            ),
        )
        incoming_message = cast(
            LedgerApiMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=ledger_api_dialogue,
                performative=LedgerApiMessage.Performative.TRANSACTION_RECEIPT,
                transaction_receipt=self.mocked_tx_receipt,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            with patch.object(LedgerApis, "is_transaction_settled", return_value=True):
                self.ledger_api_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"transaction was successfully settled. Transaction receipt={self.mocked_tx_receipt}",
        )

        assert self.strategy.contract_address == self.contract_address
        assert self.strategy.is_contract_deployed is True
        assert self.strategy.is_behaviour_active is True

    def test_handle_transaction_receipt_iii(self):
        """Test the _handle_transaction_receipt method of the ledger_api handler where is_tokens_created is False."""
        # setup
        self.strategy._is_contract_deployed = True
        self.strategy._is_tokens_created = False

        ledger_api_dialogue = cast(
            LedgerApiDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.ledger_api_dialogues,
                messages=self.list_of_ledger_api_messages[4:5],
                counterparty=LEDGER_API_ADDRESS,
            ),
        )
        incoming_message = cast(
            LedgerApiMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=ledger_api_dialogue,
                performative=LedgerApiMessage.Performative.TRANSACTION_RECEIPT,
                transaction_receipt=self.mocked_tx_receipt,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            with patch.object(LedgerApis, "is_transaction_settled", return_value=True):
                self.ledger_api_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"transaction was successfully settled. Transaction receipt={self.mocked_tx_receipt}",
        )

        assert self.strategy.is_tokens_created is True
        assert self.strategy.is_behaviour_active is True

    def test_handle_transaction_receipt_iv(self):
        """Test the _handle_transaction_receipt method of the ledger_api handler where is_tokens_minted is False."""
        # setup
        self.strategy._is_contract_deployed = True
        self.strategy._is_tokens_created = True
        self.strategy._is_tokens_minted = False

        ledger_api_dialogue = cast(
            LedgerApiDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.ledger_api_dialogues,
                messages=self.list_of_ledger_api_messages[4:5],
                counterparty=LEDGER_API_ADDRESS,
            ),
        )
        incoming_message = cast(
            LedgerApiMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=ledger_api_dialogue,
                performative=LedgerApiMessage.Performative.TRANSACTION_RECEIPT,
                transaction_receipt=self.mocked_tx_receipt,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            with patch.object(LedgerApis, "is_transaction_settled", return_value=True):
                self.ledger_api_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"transaction was successfully settled. Transaction receipt={self.mocked_tx_receipt}",
        )

        assert self.strategy.is_tokens_minted is True
        assert self.strategy.is_behaviour_active is True

    def test_handle_transaction_receipt_v(self):
        """Test the _handle_transaction_receipt method of the ledger_api handler where is_tokens_minted is True."""
        # setup
        self.strategy._is_contract_deployed = True
        self.strategy._is_tokens_created = True
        self.strategy._is_tokens_minted = True

        ledger_api_dialogue = cast(
            LedgerApiDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.ledger_api_dialogues,
                messages=self.list_of_ledger_api_messages[4:5],
                counterparty=LEDGER_API_ADDRESS,
            ),
        )
        incoming_message = cast(
            LedgerApiMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=ledger_api_dialogue,
                performative=LedgerApiMessage.Performative.TRANSACTION_RECEIPT,
                transaction_receipt=self.mocked_tx_receipt,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            with patch.object(LedgerApis, "is_transaction_settled", return_value=True):
                self.ledger_api_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"transaction was successfully settled. Transaction receipt={self.mocked_tx_receipt}",
        )

        assert self.skill.skill_context.is_active is False
        mock_logger.assert_any_call(logging.INFO, "demo finished!")

    def test_handle_error(self):
        """Test the _handle_error method of the ledger_api handler."""
        # setup
        ledger_api_dialogue = cast(
            LedgerApiDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.ledger_api_dialogues,
                messages=self.list_of_ledger_api_balance_messages[:1],
            ),
        )
        incoming_message = cast(
            LedgerApiMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=ledger_api_dialogue,
                performative=LedgerApiMessage.Performative.ERROR,
                code=1,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.ledger_api_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received ledger_api error message={incoming_message} in dialogue={ledger_api_dialogue}.",
        )

    def test_handle_invalid(self):
        """Test the _handle_invalid method of the ledger_api handler."""
        # setup
        invalid_performative = LedgerApiMessage.Performative.GET_BALANCE
        incoming_message = cast(
            LedgerApiMessage,
            self.build_incoming_message(
                message_type=LedgerApiMessage,
                dialogue_reference=("1", ""),
                performative=invalid_performative,
                ledger_id=self.ledger_id,
                address=self.address,
                to=str(self.skill.skill_context.skill_id),
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.ledger_api_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.WARNING,
            f"cannot handle ledger_api message of performative={invalid_performative} in dialogue={self.ledger_api_dialogues.get_dialogue(incoming_message)}.",
        )

    def test_teardown(self):
        """Test the teardown method of the ledger_api handler."""
        assert self.ledger_api_handler.teardown() is None
        self.assert_quantity_in_outbox(0)


class TestContractApiHandler(ERC1155DeployTestCase):
    """Test contract_api handler of erc1155_deploy."""

    is_agent_to_agent_messages = False

    def test_setup(self):
        """Test the setup method of the contract_api handler."""
        assert self.contract_api_handler.setup() is None
        self.assert_quantity_in_outbox(0)

    def test_handle_unidentified_dialogue(self):
        """Test the _handle_unidentified_dialogue method of the signing handler."""
        # setup
        incorrect_dialogue_reference = ("", "")
        incoming_message = cast(
            ContractApiMessage,
            self.build_incoming_message(
                message_type=ContractApiMessage,
                dialogue_reference=incorrect_dialogue_reference,
                performative=ContractApiMessage.Performative.STATE,
                state=State(self.ledger_id, self.body_dict),
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.contract_api_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received invalid contract_api message={incoming_message}, unidentified dialogue.",
        )

    def test_handle_raw_transaction(self):
        """Test the _handle_raw_transaction method of the signing handler."""
        # setup
        contract_api_dialogue = cast(
            ContractApiDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.contract_api_dialogues,
                messages=self.list_of_contract_api_messages[:1],
            ),
        )
        contract_api_dialogue.terms = self.mocked_terms
        incoming_message = cast(
            ContractApiMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=contract_api_dialogue,
                performative=ContractApiMessage.Performative.RAW_TRANSACTION,
                raw_transaction=self.mocked_raw_tx,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.contract_api_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO, f"received raw transaction={incoming_message}"
        )

        self.assert_quantity_in_decision_making_queue(1)
        message = self.get_message_from_decision_maker_inbox()
        has_attributes, error_str = self.message_has_attributes(
            actual_message=message,
            message_type=SigningMessage,
            performative=SigningMessage.Performative.SIGN_TRANSACTION,
            to=self.skill.skill_context.decision_maker_address,
            sender=str(self.skill.skill_context.skill_id),
            raw_transaction=self.mocked_raw_tx,
            terms=contract_api_dialogue.terms,
        )
        assert has_attributes, error_str

        assert (
            cast(
                SigningDialogue, self.signing_dialogues.get_dialogue(message)
            ).associated_contract_api_dialogue
            == contract_api_dialogue
        )

        mock_logger.assert_any_call(
            logging.INFO,
            "proposing the transaction to the decision maker. Waiting for confirmation ...",
        )

    def test_handle_error(self):
        """Test the _handle_error method of the signing handler."""
        # setup
        contract_api_dialogue = cast(
            ContractApiDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.contract_api_dialogues,
                messages=self.list_of_contract_api_messages[:1],
            ),
        )
        incoming_message = cast(
            ContractApiMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=contract_api_dialogue,
                performative=ContractApiMessage.Performative.ERROR,
                data=b"some_data",
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.contract_api_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received contract_api error message={incoming_message} in dialogue={contract_api_dialogue}.",
        )

    def test_handle_invalid(self):
        """Test the _handle_invalid method of the signing handler."""
        # setup
        invalid_performative = ContractApiMessage.Performative.GET_DEPLOY_TRANSACTION
        incoming_message = cast(
            ContractApiMessage,
            self.build_incoming_message(
                message_type=ContractApiMessage,
                dialogue_reference=("1", ""),
                performative=invalid_performative,
                ledger_id=self.ledger_id,
                contract_id=self.contract_id,
                callable=self.callable,
                kwargs=self.kwargs,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.contract_api_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.WARNING,
            f"cannot handle contract_api message of performative={invalid_performative} in dialogue={self.contract_api_dialogues.get_dialogue(incoming_message)}.",
        )

    def test_teardown(self):
        """Test the teardown method of the contract_api handler."""
        assert self.contract_api_handler.teardown() is None
        self.assert_quantity_in_outbox(0)


class TestSigningHandler(ERC1155DeployTestCase):
    """Test signing handler of erc1155_deploy."""

    is_agent_to_agent_messages = False

    def test_setup(self):
        """Test the setup method of the signing handler."""
        assert self.signing_handler.setup() is None
        self.assert_quantity_in_outbox(0)

    def test_handle_unidentified_dialogue(self):
        """Test the _handle_unidentified_dialogue method of the signing handler."""
        # setup
        incorrect_dialogue_reference = ("", "")
        incoming_message = cast(
            SigningMessage,
            self.build_incoming_message(
                message_type=SigningMessage,
                dialogue_reference=incorrect_dialogue_reference,
                performative=SigningMessage.Performative.ERROR,
                error_code=SigningMessage.ErrorCode.UNSUCCESSFUL_MESSAGE_SIGNING,
                to=str(self.skill.skill_context.skill_id),
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.signing_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received invalid signing message={incoming_message}, unidentified dialogue.",
        )

    def test_handle_signed_transaction(
        self,
    ):
        """Test the _handle_signed_transaction method of the signing handler."""
        # setup
        signing_dialogue = cast(
            SigningDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.signing_dialogues,
                messages=self.list_of_signing_messages[:1],
                counterparty=self.skill.skill_context.decision_maker_address,
            ),
        )
        incoming_message = cast(
            SigningMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=signing_dialogue,
                performative=SigningMessage.Performative.SIGNED_TRANSACTION,
                signed_transaction=self.mocked_signed_tx,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.signing_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(logging.INFO, "transaction signing was successful.")

        self.assert_quantity_in_outbox(1)
        message = self.get_message_from_outbox()
        has_attributes, error_str = self.message_has_attributes(
            actual_message=message,
            message_type=LedgerApiMessage,
            performative=LedgerApiMessage.Performative.SEND_SIGNED_TRANSACTION,
            to=LEDGER_API_ADDRESS,
            sender=str(self.skill.skill_context.skill_id),
            signed_transaction=self.mocked_signed_tx,
        )
        assert has_attributes, error_str

        ledger_api_dialogue = cast(
            LedgerApiDialogue, self.ledger_api_dialogues.get_dialogue(message)
        )
        assert ledger_api_dialogue.associated_signing_dialogue == signing_dialogue

        mock_logger.assert_any_call(logging.INFO, "sending transaction to ledger.")

    def test_handle_error(self):
        """Test the _handle_error method of the signing handler."""
        # setup
        signing_counterparty = self.skill.skill_context.decision_maker_address
        signing_dialogue = self.prepare_skill_dialogue(
            dialogues=self.signing_dialogues,
            messages=self.list_of_signing_messages[:1],
            counterparty=signing_counterparty,
        )
        incoming_message = cast(
            SigningMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=signing_dialogue,
                performative=SigningMessage.Performative.ERROR,
                error_code=SigningMessage.ErrorCode.UNSUCCESSFUL_TRANSACTION_SIGNING,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.signing_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"transaction signing was not successful. Error_code={incoming_message.error_code} in dialogue={signing_dialogue}",
        )

    def test_handle_invalid(self):
        """Test the _handle_invalid method of the signing handler."""
        # setup
        invalid_performative = SigningMessage.Performative.SIGN_TRANSACTION
        incoming_message = self.build_incoming_message(
            message_type=SigningMessage,
            dialogue_reference=("1", ""),
            performative=invalid_performative,
            terms=self.mocked_terms,
            raw_transaction=SigningMessage.RawTransaction(
                self.ledger_id, {"some_key": "some_value"}
            ),
            to=str(self.skill.skill_context.skill_id),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.signing_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.WARNING,
            f"cannot handle signing message of performative={invalid_performative} in dialogue={self.signing_dialogues.get_dialogue(incoming_message)}.",
        )

    def test_teardown(self):
        """Test the teardown method of the signing handler."""
        assert self.signing_handler.teardown() is None
        self.assert_quantity_in_outbox(0)


class TestOefSearchHandler(ERC1155DeployTestCase):
    """Test oef_search handler of erc1155_deploy."""

    is_agent_to_agent_messages = False

    def test_setup(self):
        """Test the setup method of the oef_search handler."""
        assert self.oef_search_handler.setup() is None
        self.assert_quantity_in_outbox(0)

    def test_handle_unidentified_dialogue(self):
        """Test the _handle_unidentified_dialogue method of the oef_search handler."""
        # setup
        incorrect_dialogue_reference = ("", "")
        incoming_message = cast(
            OefSearchMessage,
            self.build_incoming_message(
                message_type=OefSearchMessage,
                dialogue_reference=incorrect_dialogue_reference,
                performative=OefSearchMessage.Performative.OEF_ERROR,
                oef_error_operation=OefSearchMessage.OefErrorOperation.REGISTER_SERVICE,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.oef_search_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received invalid oef_search message={incoming_message}, unidentified dialogue.",
        )

    def test_handle_error(self):
        """Test the _handle_error method of the oef_search handler."""
        # setup
        oef_search_dialogue = cast(
            OefSearchDialogue,
            self.prepare_skill_dialogue(
                dialogues=self.oef_search_dialogues,
                messages=self.list_of_messages_register_location[:1],
            ),
        )
        incoming_message = cast(
            OefSearchMessage,
            self.build_incoming_message_for_skill_dialogue(
                dialogue=oef_search_dialogue,
                performative=OefSearchMessage.Performative.OEF_ERROR,
                oef_error_operation=OefSearchMessage.OefErrorOperation.REGISTER_SERVICE,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.oef_search_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received oef_search error message={incoming_message} in dialogue={oef_search_dialogue}.",
        )

    def test_handle_success_i(self):
        """Test the _handle_success method of the oef_search handler where the oef success targets register_service WITH location_agent data model description."""
        # setup
        oef_dialogue = self.prepare_skill_dialogue(
            dialogues=self.oef_search_dialogues,
            messages=self.list_of_messages_register_location[:1],
        )
        incoming_message = self.build_incoming_message_for_skill_dialogue(
            dialogue=oef_dialogue,
            performative=OefSearchMessage.Performative.SUCCESS,
            agents_info=OefSearchMessage.AgentsInfo({"address": {"key": "value"}}),
        )

        # operation
        with patch.object(self.oef_search_handler.context.logger, "log") as mock_logger:
            with patch.object(
                self.registration_behaviour,
                "register_service",
            ) as mock_reg:
                self.oef_search_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received oef_search success message={incoming_message} in dialogue={oef_dialogue}.",
        )
        mock_reg.assert_called_once()

    def test_handle_success_ii(self):
        """Test the _handle_success method of the oef_search handler where the oef success targets register_service WITH set_service_key data model description."""
        # setup
        oef_dialogue = self.prepare_skill_dialogue(
            dialogues=self.oef_search_dialogues,
            messages=self.list_of_messages_register_service[:1],
        )
        incoming_message = self.build_incoming_message_for_skill_dialogue(
            dialogue=oef_dialogue,
            performative=OefSearchMessage.Performative.SUCCESS,
            agents_info=OefSearchMessage.AgentsInfo({"address": {"key": "value"}}),
        )

        # operation
        with patch.object(self.oef_search_handler.context.logger, "log") as mock_logger:
            with patch.object(
                self.registration_behaviour,
                "register_genus",
            ) as mock_reg:
                self.oef_search_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received oef_search success message={incoming_message} in dialogue={oef_dialogue}.",
        )
        mock_reg.assert_called_once()

    def test_handle_success_iii(self):
        """Test the _handle_success method of the oef_search handler where the oef success targets register_service WITH personality_agent data model and genus value description."""
        # setup
        oef_dialogue = self.prepare_skill_dialogue(
            dialogues=self.oef_search_dialogues,
            messages=self.list_of_messages_register_genus[:1],
        )
        incoming_message = self.build_incoming_message_for_skill_dialogue(
            dialogue=oef_dialogue,
            performative=OefSearchMessage.Performative.SUCCESS,
            agents_info=OefSearchMessage.AgentsInfo({"address": {"key": "value"}}),
        )

        # operation
        with patch.object(self.oef_search_handler.context.logger, "log") as mock_logger:
            with patch.object(
                self.registration_behaviour,
                "register_classification",
            ) as mock_reg:
                self.oef_search_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received oef_search success message={incoming_message} in dialogue={oef_dialogue}.",
        )
        mock_reg.assert_called_once()

    def test_handle_success_iv(self):
        """Test the _handle_success method of the oef_search handler where the oef success targets register_service WITH personality_agent data model and classification value description."""
        # setup
        oef_dialogue = self.prepare_skill_dialogue(
            dialogues=self.oef_search_dialogues,
            messages=self.list_of_messages_register_classification[:1],
        )
        incoming_message = self.build_incoming_message_for_skill_dialogue(
            dialogue=oef_dialogue,
            performative=OefSearchMessage.Performative.SUCCESS,
            agents_info=OefSearchMessage.AgentsInfo({"address": {"key": "value"}}),
        )

        # operation
        with patch.object(self.oef_search_handler.context.logger, "log") as mock_logger:
            self.oef_search_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received oef_search success message={incoming_message} in dialogue={oef_dialogue}.",
        )

        assert self.registration_behaviour.is_registered is True
        assert self.registration_behaviour.registration_in_progress is False

        mock_logger.assert_any_call(
            logging.INFO,
            "the agent, with its genus and classification, and its service are successfully registered on the SOEF.",
        )

    def test_handle_success_v(self):
        """Test the _handle_success method of the oef_search handler where the oef success targets unregister_service."""
        # setup
        oef_dialogue = self.prepare_skill_dialogue(
            dialogues=self.oef_search_dialogues,
            messages=self.list_of_messages_register_invalid[:1],
        )
        incoming_message = self.build_incoming_message_for_skill_dialogue(
            dialogue=oef_dialogue,
            performative=OefSearchMessage.Performative.SUCCESS,
            agents_info=OefSearchMessage.AgentsInfo({"address": {"key": "value"}}),
        )

        # operation
        with patch.object(self.oef_search_handler.context.logger, "log") as mock_logger:
            self.oef_search_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.INFO,
            f"received oef_search success message={incoming_message} in dialogue={oef_dialogue}.",
        )
        mock_logger.assert_any_call(
            logging.WARNING,
            f"received soef SUCCESS message as a reply to the following unexpected message: {oef_dialogue.get_message_by_id(incoming_message.target)}",
        )

    def test_handle_invalid(self):
        """Test the _handle_invalid method of the oef_search handler."""
        # setup
        incoming_message = cast(
            OefSearchMessage,
            self.build_incoming_message(
                message_type=OefSearchMessage,
                performative=OefSearchMessage.Performative.REGISTER_SERVICE,
                service_description=self.mocked_proposal,
            ),
        )

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.oef_search_handler.handle(incoming_message)

        # after
        mock_logger.assert_any_call(
            logging.WARNING,
            f"cannot handle oef_search message of performative={incoming_message.performative} in dialogue={self.oef_search_dialogues.get_dialogue(incoming_message)}.",
        )

    def test_teardown(self):
        """Test the teardown method of the oef_search handler."""
        assert self.oef_search_handler.teardown() is None
        self.assert_quantity_in_outbox(0)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains test case classes based on pytest for AEA protocol testing."""

from abc import ABC, abstractmethod
from typing import List, Type
from unittest.mock import patch

import pytest

from aea.common import Address
from aea.mail.base import Envelope
from aea.protocols.base import Message
from aea.protocols.dialogue.base import Dialogue, Dialogues


class BaseProtocolMessagesTestCase(ABC):
    """Base class to test messages for the protocol."""

    @property
    @abstractmethod
    def MESSAGE_CLASS(self) -> Type[Message]:
        """Override this property in a subclass."""

    def perform_message_test(self, msg: Message) -> None:  # nosec
        """Test message encode/decode."""
        msg.to = "receiver"
        assert (
            msg._is_consistent()  # pylint: disable=protected-access
        )  # nosec - only for testing
        envelope = Envelope(to=msg.to, sender="sender", message=msg)
        envelope_bytes = envelope.encode()

        actual_envelope = Envelope.decode(envelope_bytes)
        expected_envelope = envelope

        assert expected_envelope.to == actual_envelope.to  # nosec - only for testing
        assert (
            expected_envelope.sender == actual_envelope.sender
        )  # nosec - only for testing
        assert (  # nosec - only for testing
            expected_envelope.protocol_specification_id
            == actual_envelope.protocol_specification_id
        )
        assert (
            expected_envelope.message != actual_envelope.message
        )  # nosec - only for testing

        actual_msg = self.MESSAGE_CLASS.serializer.decode(actual_envelope.message_bytes)
        actual_msg.to = actual_envelope.to
        actual_msg.sender = actual_envelope.sender
        expected_msg = msg
        assert expected_msg == actual_msg  # nosec - only for testing

    def test_messages_ok(self) -> None:
        """Run messages are ok for encode and decode."""
        for msg in self.build_messages():
            self.perform_message_test(msg)

    def test_messages_inconsistent(self) -> None:
        """Run messages are inconsistent."""
        for msg in self.build_inconsistent():
            assert (  # nosec - only for testing
                not msg._is_consistent()  # pylint: disable=protected-access
            ), msg

    def test_messages_fail_to_encode_decode(self) -> None:
        """Run messages are failing to encode and decode."""
        msg = self.build_messages()[0]
        with patch.object(msg.__class__.Performative, "__eq__", return_value=False):
            with pytest.raises(
                ValueError, match=f"Performative not valid: {msg.performative}"
            ):
                msg.serializer.encode(msg)

        encoded_msg = msg.serializer.encode(msg)
        with patch.object(msg.__class__.Performative, "__eq__", return_value=False):
            with pytest.raises(
                ValueError, match=f"Performative not valid: {msg.performative}"
            ):
                msg.serializer.decode(encoded_msg)

    @abstractmethod
    def build_messages(self) -> List[Message]:  # type: ignore[override]
        """Build the messages to be used for testing."""

    @abstractmethod
    def build_inconsistent(self) -> List[Message]:  # type: ignore[override]
        """Build inconsistent messages to be used for testing."""


class BaseProtocolDialoguesTestCase(ABC):
    """Base class to test message construction for the protocol."""

    @property
    @abstractmethod
    def MESSAGE_CLASS(self) -> Type[Message]:
        """Override this property in a subclass."""

    @property
    @abstractmethod
    def DIALOGUE_CLASS(self) -> Type[Dialogue]:
        """Override this property in a subclass."""

    @property
    @abstractmethod
    def DIALOGUES_CLASS(self) -> Type[Dialogues]:
        """Override this property in a subclass."""

    @property
    @abstractmethod
    def ROLE_FOR_THE_FIRST_MESSAGE(self) -> Dialogue.Role:
        """Override this property in a subclass."""

    def role_from_first_message(  # pylint: disable=unused-argument
        self,
        message: Message,
        receiver_address: Address,
    ) -> Dialogue.Role:
        """Infer the role of the agent from an incoming/outgoing first message

        :param message: an incoming/outgoing first message
        :param receiver_address: the address of the receiving agent
        :return: The role of the agent
        """
        return self.ROLE_FOR_THE_FIRST_MESSAGE

    def make_dialogues_class(self) -> Type[Dialogues]:
        """Make dialogues class with specific role."""

        def new_init(self_: Dialogues, self_address: Address) -> None:
            """New init function."""
            self.DIALOGUES_CLASS.__init__(  # type: ignore[call-arg]  # pylint: disable=no-value-for-parameter,unnecessary-dunder-call
                self_,
                self_address=self_address,
                role_from_first_message=self.role_from_first_message,
                dialogue_class=self.DIALOGUE_CLASS,
            )

        return type(
            f"ForTest{self.DIALOGUES_CLASS.__name__}",
            (self.DIALOGUES_CLASS,),
            {"__init__": new_init},
        )

    @abstractmethod
    def make_message_content(self) -> dict:
        """Make a dict with message contruction content for dialogues.create."""

    def test_dialogues(self) -> None:
        """Test dialogues."""
        dialogues_class = self.make_dialogues_class()
        dialogues = dialogues_class("agent_addr")  # type: ignore
        _, dialogue = dialogues.create(
            counterparty="some", **self.make_message_content()
        )
        assert dialogue is not None  # nosec - only for testing



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Project helper tools."""

import os
import pprint
import shutil
import subprocess  # nosec
import sys
import tempfile
from typing import Any, Optional

from aea.exceptions import enforce


class AEAProject:
    """A context manager class to create and delete an AEA project."""

    old_cwd: str
    temp_dir: str

    def __init__(self, name: str = "my_aea", parent_dir: Optional[str] = None):
        """
        Initialize an AEA project.

        :param name: the name of the AEA project.
        :param parent_dir: the parent directory.
        """
        self.name = name
        self.parent_dir = parent_dir

    def __enter__(self) -> None:
        """Create and enter into the project."""
        self.old_cwd = os.getcwd()
        self.temp_dir = tempfile.mkdtemp(dir=self.parent_dir)
        os.chdir(self.temp_dir)

        self.run_aea("create", "--local", "--empty", self.name, "--author", "fetchai")
        os.chdir(self.name)

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:  # type: ignore
        """Exit the context manager."""
        os.chdir(self.old_cwd)
        shutil.rmtree(self.temp_dir)

    @staticmethod
    def run_cli(*args: Any, **kwargs: Any) -> None:
        """Run a CLI command."""
        print(f"Calling command {args} with kwargs {kwargs}")
        return_code = subprocess.check_call(args, **kwargs)  # nosec
        enforce(
            return_code == 0,
            f"Return code of {pprint.pformat(args)} is {return_code} != 0.",
        )

    @classmethod
    def run_aea(cls, *args: Any, **kwargs: Any) -> None:
        """
        Run an AEA command.

        :param args: the AEA command
        :param kwargs: keyword arguments to subprocess function
        """
        cls.run_cli(sys.executable, "-m", "aea.cli", *args, **kwargs)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2023 fetchai
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test messages module for fipa protocol."""

# pylint: disable=too-many-statements,too-many-locals,no-member,too-few-public-methods,redefined-builtin
from typing import List

from aea.helpers.search.models import Constraint, ConstraintType
from aea.test_tools.test_protocol import BaseProtocolMessagesTestCase

from packages.fetchai.protocols.fipa.custom_types import Description, Query
from packages.fetchai.protocols.fipa.message import FipaMessage


class TestMessageFipa(BaseProtocolMessagesTestCase):
    """Test for the 'fipa' protocol message."""

    MESSAGE_CLASS = FipaMessage

    def build_messages(self) -> List[FipaMessage]:  # type: ignore[override]
        """Build the messages to be used for testing."""
        return [
            FipaMessage(
                performative=FipaMessage.Performative.CFP,
                query=Query([Constraint("something", ConstraintType(">", 1))]),
            ),
            FipaMessage(
                performative=FipaMessage.Performative.PROPOSE,
                proposal=Description({"foo1": 1, "bar1": 2}),
            ),
            FipaMessage(
                performative=FipaMessage.Performative.ACCEPT_W_INFORM,
                info={"some str": "some str"},
            ),
            FipaMessage(
                performative=FipaMessage.Performative.MATCH_ACCEPT_W_INFORM,
                info={"some str": "some str"},
            ),
            FipaMessage(
                performative=FipaMessage.Performative.INFORM,
                info={"some str": "some str"},
            ),
            FipaMessage(
                performative=FipaMessage.Performative.ACCEPT,
            ),
            FipaMessage(
                performative=FipaMessage.Performative.DECLINE,
            ),
            FipaMessage(
                performative=FipaMessage.Performative.MATCH_ACCEPT,
            ),
            FipaMessage(
                performative=FipaMessage.Performative.END,
            ),
        ]

    def build_inconsistent(self) -> List[FipaMessage]:  # type: ignore[override]
        """Build inconsistent messages to be used for testing."""
        return [
            FipaMessage(
                performative=FipaMessage.Performative.CFP,
                # skip content: query
            ),
            FipaMessage(
                performative=FipaMessage.Performative.PROPOSE,
                # skip content: proposal
            ),
            FipaMessage(
                performative=FipaMessage.Performative.ACCEPT_W_INFORM,
                # skip content: info
            ),
            FipaMessage(
                performative=FipaMessage.Performative.MATCH_ACCEPT_W_INFORM,
                # skip content: info
            ),
            FipaMessage(
                performative=FipaMessage.Performative.INFORM,
                # skip content: info
            ),
        ]



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Implementation of plug-in mechanism for cryptos."""

import itertools
import pprint
from importlib.metadata import EntryPoint, entry_points
from typing import Iterator, List, Set

from aea.configurations.constants import (
    ALLOWED_GROUPS,
    CRYPTO_PLUGIN_GROUP,
    DOTTED_PATH_MODULE_ELEMENT_SEPARATOR,
    FAUCET_APIS_PLUGIN_GROUP,
    LEDGER_APIS_PLUGIN_GROUP,
)
from aea.crypto import register_crypto, register_faucet_api, register_ledger_api
from aea.crypto.registries.base import EntryPoint as EntryPointString
from aea.crypto.registries.base import ItemId
from aea.exceptions import AEAException, AEAPluginError, enforce

_from_group_to_register_callable = {
    CRYPTO_PLUGIN_GROUP: register_crypto,
    LEDGER_APIS_PLUGIN_GROUP: register_ledger_api,
    FAUCET_APIS_PLUGIN_GROUP: register_faucet_api,
}


class Plugin:
    """Class that implements an AEA plugin."""

    __slots__ = ("_group", "_entry_point")

    def __init__(self, group: str, entry_point: EntryPoint):
        """
        Initialize the plugin.

        :param group: the group the plugin belongs to.
        :param entry_point: the entrypoint.
        """
        self._group = group
        self._entry_point = entry_point
        self._check_consistency()

    def _parse_entry_point(self) -> List[str]:
        """
        Parse the entry point value to extract module and attribute.

        :raises AEAPluginError: if the entry point format is invalid.
        :return: a tuple containing the module name and the attribute name.
        """
        value_parts = self._entry_point.value.split(":")
        if len(value_parts) != 2:
            raise AEAPluginError(
                f"Invalid entry point format for '{self._entry_point.name}'."
            )

        return value_parts

    def _check_consistency(self) -> None:
        """
        Check consistency of input.

        :raises AEAPluginError: if some input is not correct.  # noqa: DAR402
        """
        _error_message_prefix = f"Error with plugin '{self._entry_point.name}':"
        enforce(
            self.group in ALLOWED_GROUPS,
            f"{_error_message_prefix} '{self.group}' is not in the allowed groups: {pprint.pformat(ALLOWED_GROUPS)}",
            AEAPluginError,
        )
        enforce(
            ItemId.REGEX.match(self._entry_point.name) is not None,
            f"{_error_message_prefix} '{self._entry_point.name}' is not a valid identifier for a plugin.",
            AEAPluginError,
        )

        _, attr_path = self._parse_entry_point()
        attrs = attr_path.split(".")

        enforce(
            len(attrs) == 1,
            f"{_error_message_prefix} Nested attributes currently not supported.",
            AEAPluginError,
        )

        enforce(
            EntryPointString.REGEX.match(self.entry_point_path) is not None,
            f"{_error_message_prefix} Entry point path '{self.entry_point_path}' is not valid.",
        )

    @property
    def name(self) -> str:
        """Get the plugin identifier."""
        return self._entry_point.name

    @property
    def group(self) -> str:
        """Get the group."""
        return self._group

    @property
    def attr(self) -> str:
        """Get the class name."""
        value_parts = self._parse_entry_point()
        return value_parts[1]

    @property
    def entry_point_path(self) -> str:
        """Get the entry point path."""
        module_name, attr_name = self._parse_entry_point()
        return f"{module_name}{DOTTED_PATH_MODULE_ELEMENT_SEPARATOR}{attr_name}"


def _check_no_duplicates(plugins: List[EntryPoint]) -> None:
    """Check there are no two plugins with the same id."""
    seen: Set[str] = set()
    duplicate_plugins = [p for p in plugins if p.name in seen or seen.add(p.name)]  # type: ignore
    error_msg = f"Found plugins with the same id: {pprint.pformat(duplicate_plugins)}"
    enforce(len(duplicate_plugins) == 0, error_msg, AEAPluginError)


def _get_plugins(group: str) -> List[Plugin]:
    """
    Return a dict of all installed plugins, by name.

    :param group: the plugin group.
    :return: a mapping from plugin name to Plugin objects.
    """
    entry_points_list: List[EntryPoint] = list(entry_points(group=group))
    _check_no_duplicates(entry_points_list)
    return [Plugin(group, entry_point) for entry_point in entry_points_list]


def _get_cryptos() -> List[Plugin]:
    """Get cryptos plugins."""
    return _get_plugins(CRYPTO_PLUGIN_GROUP)


def _get_ledger_apis() -> List[Plugin]:
    """Get ledgers plugins."""
    return _get_plugins(LEDGER_APIS_PLUGIN_GROUP)


def _get_faucet_apis() -> List[Plugin]:
    """Get faucets plugins."""
    return _get_plugins(FAUCET_APIS_PLUGIN_GROUP)


def _iter_plugins() -> Iterator[Plugin]:
    """Iterate over all the plugins."""
    yield from itertools.chain(_get_cryptos(), _get_ledger_apis(), _get_faucet_apis())


def _register_plugin(plugin: Plugin, is_raising_exception: bool = True) -> None:
    """Register a plugin to the right registry."""
    register_function = _from_group_to_register_callable[plugin.group]
    try:
        register_function(plugin.name, entry_point=plugin.entry_point_path)
    except AEAException:  # pragma: nocover
        if is_raising_exception:
            raise


def load_all_plugins(is_raising_exception: bool = True) -> None:
    """Load all plugins."""
    for plugin in _iter_plugins():
        _register_plugin(plugin, is_raising_exception)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2023 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Implementation of the 'aea install' subcommand."""

from typing import Optional, Tuple, cast

import click

from aea.cli.utils.click_utils import PyPiDependency
from aea.cli.utils.context import Context
from aea.cli.utils.decorators import check_aea_project
from aea.cli.utils.loggers import logger
from aea.configurations.data_types import Dependency
from aea.exceptions import AEAException
from aea.helpers.install_dependency import call_pip, install_dependencies


@click.command()
@click.option(
    "-r",
    "--requirement",
    type=click.Path(exists=True, file_okay=True, dir_okay=False, readable=True),
    required=False,
    default=None,
    help="Install from the given requirements file.",
)
@click.option(
    "-e",
    "--extra-dependency",
    "extra_dependencies",
    type=PyPiDependency(),
    help="Provide extra dependency.",
    multiple=True,
)
@click.option(
    "--timeout",
    type=float,
    default=300.0,
    help="Specify timeout.",
)
@click.pass_context
@check_aea_project
def install(
    click_context: click.Context,
    requirement: Optional[str],
    extra_dependencies: Tuple[Dependency],
    timeout: float,
) -> None:
    """Install the dependencies of the agent."""
    ctx = cast(Context, click_context.obj)
    do_install(ctx, requirement, extra_dependencies, timeout=timeout)


def do_install(
    ctx: Context,
    requirement: Optional[str] = None,
    extra_dependencies: Optional[Tuple[Dependency]] = None,
    timeout: float = 300.0,
) -> None:
    """
    Install necessary dependencies.

    :param ctx: context object.
    :param requirement: optional str requirement.
    :param extra_dependencies: List of the extra dependencies to use
    :param timeout: timeout to wait pip to install

    :raises ClickException: if AEAException occurs.
    """
    try:
        if requirement:
            if extra_dependencies is not None and len(extra_dependencies) > 0:
                logger.debug(
                    "Extra dependencies will be ignored while installing from requirements file"
                )
            logger.debug("Installing the dependencies in '{}'...".format(requirement))
            _install_from_requirement(requirement, install_timeout=timeout)
        else:
            logger.debug("Installing all the dependencies...")
            dependencies = ctx.get_dependencies(extra_dependencies=extra_dependencies)
            install_dependencies(
                list(dependencies.values()),
                logger=logger,
                install_timeout=timeout,
            )
    except AEAException as e:
        raise click.ClickException(str(e))


def _install_from_requirement(file: str, install_timeout: float = 300) -> None:
    """
    Install from requirements.

    :param file: requirement.txt file path
    :param install_timeout: timeout to wait pip to install

    :raises AEAException: if an error occurs during installation.
    """
    try:
        call_pip(["install", "-r", file], timeout=install_timeout)
    except Exception:
        raise AEAException(
            "An error occurred while installing requirement file {}. Stopping...".format(
                file
            )
        )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2026 Valory AG
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains a test for aea.test_tools.test_protocol."""

from typing import List, Type

from aea.protocols.base import Message
from aea.protocols.dialogue.base import Dialogue as BaseDialogue
from aea.protocols.dialogue.base import Dialogues
from aea.test_tools.test_protocol import (
    BaseProtocolDialoguesTestCase,
    BaseProtocolMessagesTestCase,
)

from tests.data.packages.fetchai.protocols.t_protocol.dialogues import (
    TProtocolDialogue,
    TProtocolDialogues,
)
from tests.data.packages.fetchai.protocols.t_protocol.message import (
    CustomDataModel,
    TProtocolMessage,
)

custom_data_model = CustomDataModel(
    bool_field=True,
    bytes_field=b"",
    dict_field={},
    float_field=1.1,
    set_field=set(),
    str_field="",
    list_field=[],
    int_field=1,
)


class TestMessages(BaseProtocolMessagesTestCase):
    """Base class to test message construction for the protocol."""

    MESSAGE_CLASS = TProtocolMessage

    def build_messages(self) -> List[Message]:  # type: ignore[override]
        """Build the messages to be used for testing."""
        return [
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_CT,
                content_ct=custom_data_model,
            )
        ]

    def build_inconsistent(self) -> List[Message]:  # type: ignore[override]
        """Build inconsistent messages to be used for testing."""
        return [
            TProtocolMessage(
                performative=TProtocolMessage.Performative.PERFORMATIVE_CT,
            ),
        ]


class TestDialogues(BaseProtocolDialoguesTestCase):
    """Test dialogues."""

    MESSAGE_CLASS: Type[Message] = TProtocolMessage
    DIALOGUE_CLASS: Type[BaseDialogue] = TProtocolDialogue
    DIALOGUES_CLASS: Type[Dialogues] = TProtocolDialogues
    ROLE_FOR_THE_FIRST_MESSAGE = TProtocolDialogue.Role.ROLE_1

    def make_message_content(self) -> dict:
        """Make a dict with message contruction content for dialogues.create."""
        return dict(
            performative=TProtocolMessage.Performative.PERFORMATIVE_CT,
            content_ct=custom_data_model,
        )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test cid implementation."""

from typing import cast

import pytest

from aea.helpers.cid import CID, CIDv0, CIDv1, to_v0, to_v1

HASH_V0 = "QmbWqxBEKC3P8tqsKc98xmWNzrzDtRLMiMPL8wBuTGsMnR"
HASH_V1 = "bafybeigdyrzt5sfp7udm7hu76uh7y26nf3efuylqabf3oclgtqy55fbzdi"


def test_conversion() -> None:
    """Test conversion."""

    assert to_v0(HASH_V1) == HASH_V0
    assert to_v1(HASH_V0) == HASH_V1

    with pytest.raises(
        ValueError,
        match=f"{HASH_V0} is already v0.",
    ):
        to_v0(HASH_V0)

    with pytest.raises(
        ValueError,
        match=f"{HASH_V1} is already v1.",
    ):
        to_v1(HASH_V1)

    with pytest.raises(
        ValueError,
        match="CIDv1 can only be converted for codec dag-pb",
    ):
        cid_v1 = cast(CIDv1, CID.from_string(HASH_V1))
        cid_v1._codec = CIDv0.CODEC[::-1]
        cid_v1.to_v0()


def test_cids() -> None:
    """Test CID object."""

    cid_v0 = CID.from_string(HASH_V0)

    assert isinstance(cid_v0, CIDv0)
    assert cid_v0.version == 0
    assert str(cid_v0) == HASH_V0

    cid_v1 = CID.from_string(HASH_V1)

    assert isinstance(cid_v1, CIDv1)
    assert cid_v1.version == 1
    assert str(cid_v1) == HASH_V1


@pytest.mark.parametrize("multihash", [HASH_V0, HASH_V1])
def test_cid__eq__(multihash):
    """Test CID __eq__"""
    cid = CID.from_string(multihash)
    assert cid is not CID.from_string(multihash)
    assert cid == CID.from_string(multihash)
    assert not cid == multihash
    assert not multihash == cid


def test_make() -> None:
    """Test make method."""

    cid_v0_expected = CID.from_string(HASH_V0)
    cid_v1_expected = CID.from_string(HASH_V1)

    with pytest.raises(ValueError, match="version should be 0 or 1, 2 was provided"):
        CID.make(2, "", b"")

    with pytest.raises(ValueError, match="invalid codec  provided, please check"):
        CID.make(0, "", b"")

    with pytest.raises(
        ValueError, match="invalid type for multihash provided, should be bytes"
    ):
        CID.make(0, CIDv0.CODEC, "")  # type: ignore

    with pytest.raises(
        ValueError, match="codec for version 0 can only be dag-pb, found: protobuf"
    ):
        CID.make(0, "protobuf", b"")

    cid_v0 = CID.make(
        cid_v0_expected.version,
        cid_v0_expected.codec,
        cid_v0_expected.multihash,
    )

    assert cid_v0 == cid_v0_expected

    cid_v1 = CID.make(
        cid_v1_expected.version,
        cid_v1_expected.codec,
        cid_v1_expected.multihash,
    )

    assert cid_v1 == cid_v1_expected


@pytest.mark.parametrize("multihash", [HASH_V0, HASH_V1])
def test_cid__repr__(multihash):
    """Test CID __repr__"""

    keys = ["version", "codec", "multihash"]
    cid = CID.from_string(multihash)
    assert all(k in repr(cid) for k in keys)


@pytest.mark.parametrize("multihash", [HASH_V0, HASH_V1])
def test_is_cid(multihash):
    """Test CID.is_cid"""

    assert CID.is_cid(multihash)
    assert not CID.is_cid(multihash[::-1])


@pytest.mark.parametrize("multihash", [HASH_V0, HASH_V1])
def test_cid_from_bytes(multihash):
    """Test CID.from_bytes"""

    multihash_bytes = multihash.encode("utf-8")
    assert CID.from_bytes(multihash_bytes)


@pytest.mark.parametrize(
    "cid_bytes, error_message",
    [
        (b"b", "argument length can not be zero"),
        (b"bb", "cid length is invalid"),
        (bytes(2), "multihash is too short"),
        (bytes(10), "multihash length field does not match digest field length"),
        (
            HASH_V1[::-1].encode("utf-8"),
            "multihash is not a valid base58 encoded multihash",
        ),
    ],
)
def test_cid_from_incorrect_bytes_raises(cid_bytes, error_message):
    """Test CID.from_bytes raises on incorrect input"""

    with pytest.raises(ValueError, match=error_message):
        assert CID.from_bytes(cid_bytes)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Fixture helpers"""

from contextlib import contextmanager
from pathlib import Path
from typing import Dict

import pytest
from aea_ledger_ethereum.test_tools.constants import (
    ETHEREUM_PRIVATE_KEY_PATH,
    FUNDED_ETH_PRIVATE_KEY_1,
    FUNDED_ETH_PRIVATE_KEY_2,
    FUNDED_ETH_PRIVATE_KEY_3,
)
from aea_ledger_ethereum.test_tools.docker_images import GanacheDockerImage

from aea.test_tools.docker_image import launch_image
from aea.test_tools.network import LOCALHOST

DEFAULT_GANACHE_ADDR = LOCALHOST.geturl()
DEFAULT_GANACHE_PORT = 8545
DEFAULT_GANACHE_CHAIN_ID = 1337
DEFAULT_AMOUNT = 1000000000000000000000

GANACHE_CONFIGURATION = dict(
    accounts_balances=[
        (FUNDED_ETH_PRIVATE_KEY_1, DEFAULT_AMOUNT),
        (FUNDED_ETH_PRIVATE_KEY_2, DEFAULT_AMOUNT),
        (FUNDED_ETH_PRIVATE_KEY_3, DEFAULT_AMOUNT),
        (Path(ETHEREUM_PRIVATE_KEY_PATH).read_text().strip(), DEFAULT_AMOUNT),
    ],
)


@contextmanager
def _ganache_context(
    ganache_configuration: Dict,
    ganache_addr: str = DEFAULT_GANACHE_ADDR,
    ganache_port: int = DEFAULT_GANACHE_PORT,
    timeout: float = 2.0,
    max_attempts: int = 10,
):
    import docker  # pylint: disable=import-outside-toplevel,import-error

    client = docker.from_env()
    image = GanacheDockerImage(
        client, ganache_addr, ganache_port, config=ganache_configuration
    )
    yield from launch_image(image, timeout=timeout, max_attempts=max_attempts)


@pytest.fixture(scope="class")
def ganache(
    ganache_addr=DEFAULT_GANACHE_ADDR,
    ganache_port=DEFAULT_GANACHE_PORT,
    timeout: float = 2.0,
    max_attempts: int = 10,
):
    """Launch the Ganache image."""
    with _ganache_context(
        GANACHE_CONFIGURATION.copy(), ganache_addr, ganache_port, timeout, max_attempts
    ) as image:
        yield image



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2026 Valory AG
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Settings for operating Registry with CLI."""

from typing import Dict, Tuple

REGISTRY_CONFIG_KEY: str = "registry_config"


REGISTRY_REMOTE = "remote"
REGISTRY_LOCAL = "local"
REGISTRY_MIXED = "mixed"

REGISTRY_TYPES: Tuple[str, ...] = (REGISTRY_LOCAL, REGISTRY_REMOTE, REGISTRY_MIXED)

REMOTE_HTTP = "http"
REMOTE_IPFS = "ipfs"

REGISTRY_API_URL_KEY = "registry_api_url"
# we ignore issue B105 because this is not an hard-coded authentication token,
# but the name of the field in the configuration file.
AUTH_TOKEN_KEY = "auth_token"  # nosec

DEFAULT_IPFS_URL = "/dns/registry.autonolas.tech/tcp/443/https"
DEFAULT_IPFS_URL_LOCAL = "/ip4/127.0.0.1/tcp/5001"

DEFAULT_REGISTRY_CONFIG: Dict = {
    "default": None,
    "settings": {
        REGISTRY_REMOTE: {
            "default": None,
            REMOTE_HTTP: {
                "auth_token": None,  # auth token for registry  # nosec
                "registry_api_url": None,  # registry url
            },
            REMOTE_IPFS: {"ipfs_node": None},  # IPFS url (in multiaddr format)
        },
        REGISTRY_LOCAL: {"default_packages_path": None},
    },
}



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2026 Valory AG
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Implementation of the 'aea add' subcommand."""

import shutil
from pathlib import Path
from typing import Union, cast

import click

from aea.cli.registry.add import fetch_package
from aea.cli.registry.settings import (
    REGISTRY_LOCAL,
    REGISTRY_MIXED,
    REGISTRY_REMOTE,
    REMOTE_IPFS,
)
from aea.cli.utils.click_utils import component_flag, registry_flag
from aea.cli.utils.config import get_default_remote_registry, load_item_config
from aea.cli.utils.constants import DUMMY_PACKAGE_ID
from aea.cli.utils.context import Context
from aea.cli.utils.decorators import check_aea_project, clean_after
from aea.cli.utils.loggers import logger
from aea.cli.utils.package_utils import (
    copy_package_directory,
    find_item_in_distribution,
    find_item_locally,
    get_item_id_present,
    get_package_path,
    is_distributed_item,
    is_fingerprint_correct,
    is_item_present,
    is_item_with_hash_present,
    register_item,
)
from aea.configurations.base import (
    ConnectionConfig,
    ContractConfig,
    PackageConfiguration,
    SkillConfig,
)
from aea.configurations.constants import CONNECTION, CONTRACT, PROTOCOL, SKILL
from aea.configurations.data_types import PublicId
from aea.helpers.ipfs.base import IPFSHashOnly

try:
    from aea_cli_ipfs.exceptions import HashNotProvided  # type: ignore
    from aea_cli_ipfs.registry import fetch_ipfs  # type: ignore

    IS_IPFS_PLUGIN_INSTALLED = True
except ImportError:  # pragma: nocover
    IS_IPFS_PLUGIN_INSTALLED = False


@click.command()
@registry_flag()
@component_flag(wrap_public_id=True)
@click.pass_context
@check_aea_project
def add(
    click_context: click.Context,
    component_type: str,
    public_id: PublicId,
    registry: str,
) -> None:
    """Add a package to the agent."""
    ctx = cast(Context, click_context.obj)
    ctx.registry_type = registry
    add_item(ctx, component_type, public_id)


@clean_after
def add_item(ctx: Context, item_type: str, item_public_id: PublicId) -> None:
    """
    Add an item.

    :param ctx: Context object.
    :param item_type: the item type.
    :param item_public_id: the item public id.
    """

    from_hash = item_public_id == DUMMY_PACKAGE_ID
    present_item_id = None
    if from_hash:
        click.echo(f"Adding item from hash: {item_public_id.hash}")
        present_item_id = is_item_with_hash_present(
            ctx.cwd, ctx.agent_config, item_public_id.hash
        )
    else:
        click.echo(f"Adding {item_type} '{item_public_id}'...")
        if is_item_present(ctx.cwd, ctx.agent_config, item_type, item_public_id):
            present_item_id = get_item_id_present(
                ctx.agent_config, item_type, item_public_id
            )

    if present_item_id is not None:
        raise click.ClickException(
            "A {} with id '{}' already exists. Aborting...".format(
                item_type, present_item_id.without_hash()
            )
        )

    dest_path = get_package_path(ctx.cwd, item_type, item_public_id)
    ctx.clean_paths.append(dest_path)

    package_path = fetch_item(ctx, item_type, item_public_id, dest_path)

    if from_hash:
        (package_path_temp,) = list(package_path.parent.iterdir())
        item_config = load_item_config(item_type, package_path_temp)
        item_public_id = item_config.public_id
        package_path = Path(get_package_path(ctx.cwd, item_type, item_public_id))
        if package_path.is_dir():
            shutil.rmtree(package_path_temp.parent.parent)
            raise click.ClickException(
                f"Package with id {item_public_id} already exists."
            )
        ctx.clean_paths.append(package_path)
        shutil.move(str(package_path_temp), package_path)

    item_config = load_item_config(item_type, package_path)
    if not ctx.config.get("skip_consistency_check") and not is_fingerprint_correct(
        package_path, item_config
    ):  # pragma: no cover
        raise click.ClickException("Failed to add an item with incorrect fingerprint.")
    _add_item_deps(ctx, item_type, item_config)

    try:
        package_hash = item_public_id.hash
    except (ValueError, AttributeError):
        package_hash = IPFSHashOnly().hash_directory(str(package_path))

    register_item(
        ctx,
        item_type,
        PublicId.from_json(
            {**item_config.public_id.json, "package_hash": package_hash}
        ),
    )
    click.echo(f"Successfully added {item_type} '{item_config.public_id}'.")


def fetch_item(
    ctx: Context, item_type: str, item_public_id: PublicId, dest_path: str
) -> Path:
    """
    Unify fetch item locally, remotely or mixed.

    :param ctx: the CLI context.
    :param item_type: the item type.
    :param item_public_id: the item public id.
    :param dest_path: the path to the destination.
    :return: the path to the found package.
    """

    if ctx.registry_type == REGISTRY_LOCAL:
        package_path = find_item_locally_or_distributed(
            ctx, item_type, item_public_id, dest_path
        )
    elif ctx.registry_type == REGISTRY_REMOTE:
        package_path = fetch_item_remote(ctx, item_type, item_public_id, dest_path)
    elif ctx.registry_type == REGISTRY_MIXED:
        package_path = fetch_item_mixed(ctx, item_type, item_public_id, dest_path)
    else:  # pragma: nocover
        raise ValueError(f"Unknown registry type: {ctx.registry_type}")
    return package_path


def _add_item_deps(
    ctx: Context, item_type: str, item_config: PackageConfiguration
) -> None:
    """
    Add item dependencies. Calls add_item recursively.

    :param ctx: Context object.
    :param item_type: type of item.
    :param item_config: item configuration object.
    """
    if item_type in {CONNECTION, SKILL}:
        item_config = cast(Union[SkillConfig, ConnectionConfig], item_config)
        # add missing protocols
        for protocol_public_id in item_config.protocols:
            if protocol_public_id not in ctx.agent_config.protocols:
                add_item(ctx, PROTOCOL, protocol_public_id)

    if item_type == SKILL:
        item_config = cast(SkillConfig, item_config)
        # add missing contracts
        for contract_public_id in item_config.contracts:
            if contract_public_id not in ctx.agent_config.contracts:
                add_item(ctx, CONTRACT, contract_public_id)

        # add missing connections
        for connection_public_id in item_config.connections:
            if connection_public_id not in ctx.agent_config.connections:
                add_item(ctx, CONNECTION, connection_public_id)

        # add missing skill
        for skill_public_id in item_config.skills:
            if skill_public_id not in ctx.agent_config.skills:
                add_item(ctx, SKILL, skill_public_id)

    if item_type == CONTRACT:
        item_config = cast(ContractConfig, item_config)
        # add missing contracts
        for contract_public_id in item_config.contracts:
            if contract_public_id not in ctx.agent_config.contracts:
                add_item(ctx, CONTRACT, contract_public_id)


def fetch_item_remote(
    ctx: Context, item_type: str, item_public_id: PublicId, dest_path: str
) -> Path:
    """Fetch item remote."""
    if get_default_remote_registry() == REMOTE_IPFS:
        try:
            return cast(Path, fetch_ipfs(item_type, item_public_id, dest_path))
        except HashNotProvided:
            click.echo(f"Hash was not provided for: {item_public_id}")
            click.echo("Will try with http repository.")

    return fetch_package(
        item_type,
        public_id=item_public_id,
        cwd=ctx.cwd,
        dest=dest_path,
    )


def find_item_locally_or_distributed(
    ctx: Context, item_type: str, item_public_id: PublicId, dest_path: str
) -> Path:
    """
    Unify find item locally both in case it is distributed or not.

    :param ctx: the CLI context.
    :param item_type: the item type.
    :param item_public_id: the item public id.
    :param dest_path: the path to the destination.
    :return: the path to the found package.
    """
    is_distributed = is_distributed_item(item_public_id)
    if is_distributed:  # pragma: nocover
        source_path = find_item_in_distribution(ctx, item_type, item_public_id)
        package_path = copy_package_directory(source_path, dest_path)
    else:
        source_path, _ = find_item_locally(ctx, item_type, item_public_id)
        package_path = copy_package_directory(source_path, dest_path)
    return package_path


def fetch_item_mixed(
    ctx: Context,
    item_type: str,
    item_public_id: PublicId,
    dest_path: str,
) -> Path:
    """
    Find item, mixed mode.That is, give priority to local registry, and fall back to remote registry in case of failure.

    :param ctx: the CLI context.
    :param item_type: the item type.
    :param item_public_id: the item public id.
    :param dest_path: the path to the destination.
    :return: the path to the found package.
    """
    try:
        package_path = find_item_locally_or_distributed(
            ctx, item_type, item_public_id, dest_path
        )
    except click.ClickException as e:
        logger.debug(
            f"Fetch from local registry failed (reason={str(e)}), trying remote registry..."
        )
        # the following might raise exception, but we don't catch it this time
        package_path = fetch_item_remote(
            item_type=item_type,
            item_public_id=item_public_id,
            ctx=ctx,
            dest_path=dest_path,
        )
    return package_path



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This test module contains tests for P2PLibp2p connection."""

# pylint: skip-file

import re
from unittest.mock import Mock

import pytest
import requests

from aea.mail.base import Envelope

from packages.fetchai.protocols.default.message import DefaultMessage
from packages.valory.connections.p2p_libp2p_mailbox.connection import NodeClient
from packages.valory.connections.test_libp2p.tests.base import BaseP2PLibp2pTest, ports
from packages.valory.protocols.acn import acn_pb2
from packages.valory.protocols.acn.message import AcnMessage

MockDefaultMessageProtocol = Mock()
MockDefaultMessageProtocol.protocol_id = DefaultMessage.protocol_id
MockDefaultMessageProtocol.protocol_specification_id = (
    DefaultMessage.protocol_specification_id
)


@pytest.mark.asyncio
class TestMailboxAPI(BaseP2PLibp2pTest):
    """Test that connection is established and torn down correctly"""

    @classmethod
    def setup_class(cls):
        """Set the test up"""
        super().setup_class()

        cls.delegate_port = next(ports)
        cls.mailbox_port = next(ports)

        cls.connection_node = cls.make_connection(
            delegate=True,
            delegate_port=cls.delegate_port,
            mailbox=True,
            mailbox_port=cls.mailbox_port,
        )

        cls.connection_client = cls.make_client_connection(
            peer_public_key=cls.connection_node.node.pub,
            node_port=cls.delegate_port,
        )

    @pytest.mark.asyncio
    async def test_message_delivery(self):  # nosec
        """Test connect then disconnect."""

        url = f"https://localhost:{self.mailbox_port}"
        r = requests.get(f"{url}/ssl_signature", verify=False)  # nosec
        assert r.status_code == 200, r.text

        node_client = NodeClient(Mock(), self.connection_client.node_por)
        agent_record = node_client.make_agent_record()
        addr = agent_record.address
        performative = acn_pb2.AcnMessage.Register_Performative()  # type: ignore
        AcnMessage.AgentRecord.encode(
            performative.record, agent_record  # pylint: disable=no-member
        )
        data = performative.record.SerializeToString()  # pylint: disable=no-member

        r = requests.post(f"{url}/register", data=data, verify=False)  # nosec
        assert r.status_code == 200, r.text
        assert re.match(
            "[0-9a-f]{32}", r.text, re.I
        ), r.text  # pylint: disable=no-member
        session_id = r.text

        envelope = self.enveloped_default_message(to=addr, sender=addr)
        r = requests.post(
            f"{url}/send_envelope",
            data=envelope.encode(),
            headers={"Session-Id": session_id},
            verify=False,  # nosec
        )
        assert r.status_code == 200, r.text

        r = requests.get(
            f"{url}/get_envelope",
            headers={"Session-Id": session_id},
            verify=False,  # nosec
        )
        assert r.status_code == 200, r.text
        assert r.content, "no envelope"

        delivered_envelope = Envelope.decode(r.content)
        self.sent_is_delivered_envelope(envelope, delivered_envelope)

        # no new envelopes
        r = requests.get(
            f"{url}/get_envelope",
            headers={"Session-Id": session_id},
            verify=False,  # nosec
        )
        assert r.status_code == 200, r.text
        assert not r.content

        # unregister
        r = requests.get(
            f"{url}/unregister",
            headers={"Session-Id": session_id},
            verify=False,  # nosec
        )
        assert r.status_code == 200, r.text

        # bad session!
        r = requests.get(
            f"{url}/get_envelope",
            headers={"Session-Id": session_id},
            verify=False,  # nosec
        )
        assert r.status_code == 400, r.text
        assert "session_id" in r.text



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ledger_api.proto
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x10ledger_api.proto\x12\x1c\x61\x65\x61.valory.ledger_api.v1_0_0"\xf1\x1b\n\x10LedgerApiMessage\x12V\n\x07\x62\x61lance\x18\x05 \x01(\x0b\x32\x43.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Balance_PerformativeH\x00\x12R\n\x05\x65rror\x18\x06 \x01(\x0b\x32\x41.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Error_PerformativeH\x00\x12^\n\x0bget_balance\x18\x07 \x01(\x0b\x32G.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Get_Balance_PerformativeH\x00\x12n\n\x13get_raw_transaction\x18\x08 \x01(\x0b\x32O.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Get_Raw_Transaction_PerformativeH\x00\x12Z\n\tget_state\x18\t \x01(\x0b\x32\x45.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Get_State_PerformativeH\x00\x12v\n\x17get_transaction_receipt\x18\n \x01(\x0b\x32S.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Get_Transaction_Receipt_PerformativeH\x00\x12\x66\n\x0fraw_transaction\x18\x0b \x01(\x0b\x32K.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Raw_Transaction_PerformativeH\x00\x12v\n\x17send_signed_transaction\x18\x0c \x01(\x0b\x32S.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Send_Signed_Transaction_PerformativeH\x00\x12x\n\x18send_signed_transactions\x18\r \x01(\x0b\x32T.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Send_Signed_Transactions_PerformativeH\x00\x12R\n\x05state\x18\x0e \x01(\x0b\x32\x41.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.State_PerformativeH\x00\x12l\n\x12transaction_digest\x18\x0f \x01(\x0b\x32N.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Transaction_Digest_PerformativeH\x00\x12n\n\x13transaction_digests\x18\x10 \x01(\x0b\x32O.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Transaction_Digests_PerformativeH\x00\x12n\n\x13transaction_receipt\x18\x11 \x01(\x0b\x32O.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Transaction_Receipt_PerformativeH\x00\x1a\x18\n\x06Kwargs\x12\x0e\n\x06kwargs\x18\x01 \x01(\x0c\x1a)\n\x0eRawTransaction\x12\x17\n\x0fraw_transaction\x18\x01 \x01(\x0c\x1a/\n\x11SignedTransaction\x12\x1a\n\x12signed_transaction\x18\x01 \x01(\x0c\x1a\x44\n\x12SignedTransactions\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x1b\n\x13signed_transactions\x18\x02 \x03(\x0c\x1a\x16\n\x05State\x12\r\n\x05state\x18\x01 \x01(\x0c\x1a\x16\n\x05Terms\x12\r\n\x05terms\x18\x01 \x01(\x0c\x1a/\n\x11TransactionDigest\x12\x1a\n\x12transaction_digest\x18\x01 \x01(\x0c\x1a\x44\n\x12TransactionDigests\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x1b\n\x13transaction_digests\x18\x02 \x03(\t\x1a\x31\n\x12TransactionReceipt\x12\x1b\n\x13transaction_receipt\x18\x01 \x01(\x0c\x1a>\n\x18Get_Balance_Performative\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x0f\n\x07\x61\x64\x64ress\x18\x02 \x01(\t\x1ag\n Get_Raw_Transaction_Performative\x12\x43\n\x05terms\x18\x01 \x01(\x0b\x32\x34.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Terms\x1a\xcb\x01\n$Send_Signed_Transaction_Performative\x12\\\n\x12signed_transaction\x18\x01 \x01(\x0b\x32@.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.SignedTransaction\x12\x45\n\x06kwargs\x18\x02 \x01(\x0b\x32\x35.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Kwargs\x1a\xce\x01\n%Send_Signed_Transactions_Performative\x12^\n\x13signed_transactions\x18\x01 \x01(\x0b\x32\x41.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.SignedTransactions\x12\x45\n\x06kwargs\x18\x02 \x01(\x0b\x32\x35.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Kwargs\x1a\xf0\x01\n$Get_Transaction_Receipt_Performative\x12\\\n\x12transaction_digest\x18\x01 \x01(\x0b\x32@.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.TransactionDigest\x12\x15\n\rretry_timeout\x18\x02 \x01(\x05\x12\x1c\n\x14retry_timeout_is_set\x18\x03 \x01(\x08\x12\x16\n\x0eretry_attempts\x18\x04 \x01(\x05\x12\x1d\n\x15retry_attempts_is_set\x18\x05 \x01(\x08\x1a:\n\x14\x42\x61lance_Performative\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x0f\n\x07\x62\x61lance\x18\x02 \x01(\x05\x1av\n\x1cRaw_Transaction_Performative\x12V\n\x0fraw_transaction\x18\x01 \x01(\x0b\x32=.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.RawTransaction\x1a\x7f\n\x1fTransaction_Digest_Performative\x12\\\n\x12transaction_digest\x18\x01 \x01(\x0b\x32@.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.TransactionDigest\x1a\x82\x01\n Transaction_Digests_Performative\x12^\n\x13transaction_digests\x18\x01 \x01(\x0b\x32\x41.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.TransactionDigests\x1a\x82\x01\n Transaction_Receipt_Performative\x12^\n\x13transaction_receipt\x18\x01 \x01(\x0b\x32\x41.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.TransactionReceipt\x1a\x92\x01\n\x16Get_State_Performative\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x10\n\x08\x63\x61llable\x18\x02 \x01(\t\x12\x0c\n\x04\x61rgs\x18\x03 \x03(\t\x12\x45\n\x06kwargs\x18\x04 \x01(\x0b\x32\x35.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.Kwargs\x1al\n\x12State_Performative\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x43\n\x05state\x18\x02 \x01(\x0b\x32\x34.aea.valory.ledger_api.v1_0_0.LedgerApiMessage.State\x1an\n\x12\x45rror_Performative\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x16\n\x0emessage_is_set\x18\x03 \x01(\x08\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\x0c\x12\x13\n\x0b\x64\x61ta_is_set\x18\x05 \x01(\x08\x42\x0e\n\x0cperformativeb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "ledger_api_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_LEDGERAPIMESSAGE"]._serialized_start = 51
    _globals["_LEDGERAPIMESSAGE"]._serialized_end = 3620
    _globals["_LEDGERAPIMESSAGE_KWARGS"]._serialized_start = 1427
    _globals["_LEDGERAPIMESSAGE_KWARGS"]._serialized_end = 1451
    _globals["_LEDGERAPIMESSAGE_RAWTRANSACTION"]._serialized_start = 1453
    _globals["_LEDGERAPIMESSAGE_RAWTRANSACTION"]._serialized_end = 1494
    _globals["_LEDGERAPIMESSAGE_SIGNEDTRANSACTION"]._serialized_start = 1496
    _globals["_LEDGERAPIMESSAGE_SIGNEDTRANSACTION"]._serialized_end = 1543
    _globals["_LEDGERAPIMESSAGE_SIGNEDTRANSACTIONS"]._serialized_start = 1545
    _globals["_LEDGERAPIMESSAGE_SIGNEDTRANSACTIONS"]._serialized_end = 1613
    _globals["_LEDGERAPIMESSAGE_STATE"]._serialized_start = 1615
    _globals["_LEDGERAPIMESSAGE_STATE"]._serialized_end = 1637
    _globals["_LEDGERAPIMESSAGE_TERMS"]._serialized_start = 1639
    _globals["_LEDGERAPIMESSAGE_TERMS"]._serialized_end = 1661
    _globals["_LEDGERAPIMESSAGE_TRANSACTIONDIGEST"]._serialized_start = 1663
    _globals["_LEDGERAPIMESSAGE_TRANSACTIONDIGEST"]._serialized_end = 1710
    _globals["_LEDGERAPIMESSAGE_TRANSACTIONDIGESTS"]._serialized_start = 1712
    _globals["_LEDGERAPIMESSAGE_TRANSACTIONDIGESTS"]._serialized_end = 1780
    _globals["_LEDGERAPIMESSAGE_TRANSACTIONRECEIPT"]._serialized_start = 1782
    _globals["_LEDGERAPIMESSAGE_TRANSACTIONRECEIPT"]._serialized_end = 1831
    _globals["_LEDGERAPIMESSAGE_GET_BALANCE_PERFORMATIVE"]._serialized_start = 1833
    _globals["_LEDGERAPIMESSAGE_GET_BALANCE_PERFORMATIVE"]._serialized_end = 1895
    _globals["_LEDGERAPIMESSAGE_GET_RAW_TRANSACTION_PERFORMATIVE"]._serialized_start = (
        1897
    )
    _globals["_LEDGERAPIMESSAGE_GET_RAW_TRANSACTION_PERFORMATIVE"]._serialized_end = (
        2000
    )
    _globals[
        "_LEDGERAPIMESSAGE_SEND_SIGNED_TRANSACTION_PERFORMATIVE"
    ]._serialized_start = 2003
    _globals[
        "_LEDGERAPIMESSAGE_SEND_SIGNED_TRANSACTION_PERFORMATIVE"
    ]._serialized_end = 2206
    _globals[
        "_LEDGERAPIMESSAGE_SEND_SIGNED_TRANSACTIONS_PERFORMATIVE"
    ]._serialized_start = 2209
    _globals[
        "_LEDGERAPIMESSAGE_SEND_SIGNED_TRANSACTIONS_PERFORMATIVE"
    ]._serialized_end = 2415
    _globals[
        "_LEDGERAPIMESSAGE_GET_TRANSACTION_RECEIPT_PERFORMATIVE"
    ]._serialized_start = 2418
    _globals[
        "_LEDGERAPIMESSAGE_GET_TRANSACTION_RECEIPT_PERFORMATIVE"
    ]._serialized_end = 2658
    _globals["_LEDGERAPIMESSAGE_BALANCE_PERFORMATIVE"]._serialized_start = 2660
    _globals["_LEDGERAPIMESSAGE_BALANCE_PERFORMATIVE"]._serialized_end = 2718
    _globals["_LEDGERAPIMESSAGE_RAW_TRANSACTION_PERFORMATIVE"]._serialized_start = 2720
    _globals["_LEDGERAPIMESSAGE_RAW_TRANSACTION_PERFORMATIVE"]._serialized_end = 2838
    _globals["_LEDGERAPIMESSAGE_TRANSACTION_DIGEST_PERFORMATIVE"]._serialized_start = (
        2840
    )
    _globals["_LEDGERAPIMESSAGE_TRANSACTION_DIGEST_PERFORMATIVE"]._serialized_end = 2967
    _globals["_LEDGERAPIMESSAGE_TRANSACTION_DIGESTS_PERFORMATIVE"]._serialized_start = (
        2970
    )
    _globals["_LEDGERAPIMESSAGE_TRANSACTION_DIGESTS_PERFORMATIVE"]._serialized_end = (
        3100
    )
    _globals["_LEDGERAPIMESSAGE_TRANSACTION_RECEIPT_PERFORMATIVE"]._serialized_start = (
        3103
    )
    _globals["_LEDGERAPIMESSAGE_TRANSACTION_RECEIPT_PERFORMATIVE"]._serialized_end = (
        3233
    )
    _globals["_LEDGERAPIMESSAGE_GET_STATE_PERFORMATIVE"]._serialized_start = 3236
    _globals["_LEDGERAPIMESSAGE_GET_STATE_PERFORMATIVE"]._serialized_end = 3382
    _globals["_LEDGERAPIMESSAGE_STATE_PERFORMATIVE"]._serialized_start = 3384
    _globals["_LEDGERAPIMESSAGE_STATE_PERFORMATIVE"]._serialized_end = 3492
    _globals["_LEDGERAPIMESSAGE_ERROR_PERFORMATIVE"]._serialized_start = 3494
    _globals["_LEDGERAPIMESSAGE_ERROR_PERFORMATIVE"]._serialized_end = 3604
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This test module contains AEA cli tests for Libp2p tcp client connection."""

# pylint: skip-file

import json
from pathlib import Path

from aea.configurations.constants import DEFAULT_LEDGER
from aea.multiplexer import Multiplexer
from aea.test_tools.network import LOCALHOST
from aea.test_tools.test_cases import AEATestCaseEmpty

from packages.valory.connections import p2p_libp2p_client
from packages.valory.connections.p2p_libp2p.tests.base import libp2p_log_on_failure_all
from packages.valory.connections.p2p_libp2p_client.connection import PUBLIC_ID
from packages.valory.connections.test_libp2p.tests.base import (
    SKIP_WINDOWS,
    _make_libp2p_connection,
    make_cert_request,
    ports,
)

DEFAULT_HOST = LOCALHOST.hostname
DEFAULT_CLIENTS_PER_NODE = 4
DEFAULT_LAUNCH_TIMEOUT = 10


@SKIP_WINDOWS
@libp2p_log_on_failure_all
class TestP2PLibp2pClientConnectionAEARunning(AEATestCaseEmpty):
    """Test AEA with client connection is correctly run"""

    conn_path = f"vendor.{p2p_libp2p_client.__name__.split('.', 1)[-1]}"
    public_id = str(PUBLIC_ID)
    port = next(ports)
    uri = f"{DEFAULT_HOST}:{port}"
    kwargs = dict(
        delegate_host=DEFAULT_HOST,
        delegate_port=port,
        delegate=True,
    )
    package_registry_src_rel: Path = Path(
        __file__
    ).parent.parent.parent.parent.parent.parent

    @classmethod
    def setup_class(cls):
        """Set up the test class."""
        super().setup_class()
        cls.node_connection = _make_libp2p_connection(**cls.kwargs)
        cls.node_multiplexer = Multiplexer([cls.node_connection])
        cls.log_files = [cls.node_connection.node.log_file]
        cls.node_multiplexer.connect()
        node = {"uri": cls.uri, "public_key": cls.node_connection.node.pub}
        cls.nodes = {"nodes": [node]}
        cls.expected = (f"Successfully connected to libp2p node {cls.uri}",)

    def test_node(self):
        """Test the node is connected."""
        assert self.node_connection.is_connected is True

    def test_connection(self):
        """Test the connection can be used in an aea."""
        ledger_id = DEFAULT_LEDGER
        self.generate_private_key(ledger_id)
        self.add_private_key(ledger_id, f"{ledger_id}_private_key.txt")
        self.set_config("agent.default_ledger", ledger_id)
        self.set_config("agent.required_ledgers", json.dumps([ledger_id]), "list")
        self.add_item("connection", self.public_id)
        self.nested_set_config(self.conn_path + ".config", self.nodes)

        # generate certificates for connection
        self.nested_set_config(
            self.conn_path + ".cert_requests",
            [make_cert_request(self.node_connection.node.pub, ledger_id, "./cli_test")],
        )
        self.run_cli_command("issue-certificates", cwd=self._get_cwd())

        process = self.run_agent()
        is_running = self.is_running(process, timeout=DEFAULT_LAUNCH_TIMEOUT)
        assert is_running, "AEA not running within timeout!"
        missing_strings = self.missing_from_output(process, self.expected)
        assert not missing_strings

    @classmethod
    def teardown_class(cls):
        """Tear down the test"""
        cls.node_multiplexer.disconnect()
        super().teardown_class()



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""
This module contains an adaptation of click.testing.CliRunner.

In particular, it fixes two issues with CliRunner.invoke

1. in the 'finally' clause. More precisely, before reading from
   the testing outstream, it checks whether it has been already closed.

    Links:
    - https://github.com/pallets/click/issues/824

2. it allows for optional usage of pytest capfd, which more reliably captures byte streams

    Links:
    - https://github.com/valory-xyz/open-aea/issues/171
    - https://github.com/valory-xyz/open-aea/issues/353
"""

import os
import shlex
import subprocess  # nosec
import sys
import tempfile
from abc import ABC
from contextlib import nullcontext
from pathlib import Path
from typing import Any, ContextManager, Optional, Sequence, Union, cast

import click.core
import pytest
from _pytest.capture import CaptureFixture  # type: ignore
from click.testing import CliRunner as ClickCliRunner
from click.testing import Result

from aea.cli import cli as aea_cli
from aea.test_tools.utils import remove_test_directory


class CliRunner(ClickCliRunner):
    """Patch of click.testing.CliRunner."""

    # NOTE: always set on the instance, never the class
    capfd: Optional[CaptureFixture] = None

    def invoke(  # type: ignore  # pylint: disable=too-many-positional-arguments
        self,
        cli,
        args=None,
        input=None,  # pylint: disable=redefined-builtin
        env=None,
        catch_exceptions=True,
        color=False,
        **extra,
    ) -> Result:
        """Call a cli command with click.testing.CliRunner.invoke."""
        exc_info = None
        exception: Optional[BaseException] = None
        exit_code: Union[str, int, None] = 0

        if self.capfd:
            if input or env or color:
                raise NotImplementedError(
                    """Cannot use capfd in conjunction with `input`, `env` or `color`.
                If you wish to enable their usage with capfd, you may patch the
                CliRunner.isolation context manager. Three lines of code to set
                back the newly assigned sys.stdin, sys.stdout and sys.stderr
                immediately after they get initially re-assigned likely suffices"""
                )
            cm = nullcontext()
        else:
            cm = self.isolation(input=input, env=env, color=color)

        with cast(ContextManager, cm) as outstreams:
            if isinstance(args, str):
                args = shlex.split(args)

            try:
                prog_name = extra.pop("prog_name")
            except KeyError:
                prog_name = self.get_default_prog_name(cli)

            try:
                cli.main(args=args or (), prog_name=prog_name, **extra)
            except SystemExit as e:
                exc_info = sys.exc_info()
                exit_code = e.code
                if exit_code is None:  # pragma: nocover
                    exit_code = 0

                if exit_code != 0:  # pragma: nocover
                    exception = e

                if not isinstance(exit_code, int):
                    sys.stdout.write(str(exit_code))
                    sys.stdout.write("\n")
                    exit_code = 1

            except Exception as e:  # pylint: disable=broad-except
                if not catch_exceptions:
                    raise
                exception = e
                exit_code = 1
                exc_info = sys.exc_info()
            finally:
                if self.capfd:
                    out, err = self.capfd.readouterr()
                    stdout = out.encode()
                    stderr = err.encode()
                else:
                    sys.stdout.flush()
                    stdout = outstreams[0].getvalue() if not outstreams[0].closed else b""  # type: ignore
                    stderr = (
                        outstreams[1].getvalue() if not outstreams[1].closed else b""  # type: ignore
                    )

        return Result(
            runner=self,
            stdout_bytes=stdout,
            stderr_bytes=stderr,  # type: ignore
            output_bytes=stdout + stderr,
            exit_code=exit_code,
            exception=exception,
            exc_info=exc_info,  # type: ignore
            return_value=None,
        )


class CliTest(ABC):
    """Test cli commands."""

    cli_options: Sequence[str] = ()

    _t: Path
    __cwd: Path
    __cli: click.core.Group = aea_cli
    __cli_runner: CliRunner

    @pytest.fixture(autouse=True)
    def set_capfd_on_cli_runner(self, capfd: CaptureFixture) -> None:
        """Set pytest capfd on CLI runner"""

        self.__cli_runner.capfd = capfd

    @property
    def t(self) -> Path:
        """Get t."""
        return self._t

    @classmethod
    def setup_class(cls) -> None:
        """Setup test class."""

        cls.__cli_runner = CliRunner()
        cls.__cwd = Path.cwd().absolute()

    @classmethod
    def teardown_class(cls) -> None:
        """Teardown test class."""

        os.chdir(cls.__cwd)

    def setup_method(self) -> None:
        """Setup test."""

        self._t = Path(tempfile.mkdtemp()).resolve()

    def teardown_method(self) -> None:
        """Teardown test."""

        remove_test_directory(self._t)

    def run_cli(
        self,
        *commands: str,
        **kwargs: Any,
    ) -> Result:
        """Run CLI."""

        args = (*self.cli_options, *commands)
        return self.__cli_runner.invoke(cli=self.__cli, args=args, **kwargs)

    def run_cli_subprocess(
        self,
        *commands: str,
        timeout: float = 60.0,
        **kwargs: Any,
    ) -> Result:
        """Run CLI using subprocess."""

        cli_name = f"{self.__cli.name}.cli"  # pylint: disable=no-member
        args = (*self.cli_options, *commands)
        stderr = subprocess.PIPE
        process = subprocess.Popen(  # nosec
            [sys.executable, "-m", cli_name, *args],
            **kwargs,
            stdout=subprocess.PIPE,
            stderr=stderr,
        )

        stdout_bytes, stderr_bytes = process.communicate(timeout=timeout)

        return Result(
            runner=self.__cli_runner,
            stdout_bytes=stdout_bytes,
            stderr_bytes=stderr_bytes,
            output_bytes=stdout_bytes + stderr_bytes,
            exit_code=process.returncode,
            exception=None,
            exc_info=None,
            return_value=None,
        )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2026 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Helper utils."""

import socket
from urllib.parse import urlparse

LOCALHOST = urlparse("http://127.0.0.1")


def get_unused_tcp_port() -> int:
    """Get an unused TCP port."""
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind((LOCALHOST.hostname, 0))
    s.listen(1)
    port = s.getsockname()[1]
    s.close()
    return port


def get_host() -> str:
    """Get the host."""
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        # doesn't even have to be reachable
        s.connect(("10.255.255.255", 1))
        IP = s.getsockname()[0]
    except Exception:  # pylint: disable=broad-except
        IP = LOCALHOST.hostname
    finally:
        s.close()
    return IP


def is_port_in_use(port: int) -> bool:
    """Check if port is in use"""
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        return s.connect_ex(("localhost", port)) == 0
